<?php
if ( !class_exists( 'pluginbuddy_thumbsup_admin' ) ) {
	class pluginbuddy_thumbsup_admin {
		private $parent;
		private $menu;
		
		function __construct( &$parent ) {
			$this->parent = $parent;
			$this->menu = array(
				'addnew' => array( 
					'title' => __( 'Add New Group', 'it-l10n-thumbsup' ),
					'callback' => 'view_add_new',
					'script_handlers' => array( 'jquery-ui-timepicker' ),
					'style_handlers' => array( 'jquery-ui-timepicker' )
				), 
				'edit' => array(
					'title' => __( 'Edit Groups', 'it-l10n-thumbsup' ),
					'callback' => 'view_edit_items',
					'script_handlers' => array(),
					'style_handlers' => array()
				),
				'stats' => array(
					'title' => __( 'Group Stats', 'it-l10n-thumbsup' ),
					'callback' => 'view_stats',
					'script_handlers' => array(),
					'style_handlers' => array()
				),
				'settings' => array(
					'title' => __( 'Group Settings', 'it-l10n-thumbsup' ),
					'callback' => 'view_settings',
					'script_handlers' => array(),
					'style_handlers' => array()
				),
				'post_settings' => array(
					'title' => __( 'Post Settings', 'it-l10n-thumbsup' ),
					'callback' => 'view_post_settings',
					'script_handlers' => array(),
					'style_handlers' => array()
				),
				'post_stats' => array(
					'title' => __( 'Post Stats', 'it-l10n-thumbsup' ),
					'callback' => 'view_post_stats',
					'script_handlers' => array(),
					'style_handlers' => array()
				) 
			);
			add_action( 'admin_menu', array( &$this, 'admin_menu' ) ); // Add menu in admin.
			
			//Register script handlers
			//todo - compress this script
			wp_register_script( 'jquery-ui-timepicker', $this->get_plugin_url( 'js/jquery.ui.timepicker.js' ), array( 'jquery-ui-core' ) );
			wp_register_script( 'jquery-ui-datepicker', $this->get_plugin_url( 'js/jquery-ui-datepicker.min.js' ), array( 'jquery-ui-core') );
			//Register style handlers
			wp_register_style( 'jquery-ui-timepicker', $this->get_plugin_url( 'css/jquery-ui-timepicker.css' ) );
			wp_register_style( 'jquery-ui-datepicker', $this->get_plugin_url( 'css/ui-lightness/jquery-ui-datepicker.css' ) );
			
			//Add Ajax actions
			add_action( 'wp_ajax_submit_voting_group', array( &$this, 'ajax_submit_voting_group' ) );
			add_action( 'wp_ajax_submit_voting_item', array( &$this, 'ajax_submit_voting_item' ) );
			add_action( 'wp_ajax_approve_voting_item', array( &$this, 'ajax_approve_voting_item' ) );
			add_action( 'wp_ajax_delete_voting_item', array( &$this, 'ajax_delete_voting_item' ) );
			add_action( 'wp_ajax_edit_voting_item', array( &$this, 'ajax_edit_voting_item' ) );
			add_action( 'wp_ajax_sort_voting_items', array( &$this, 'ajax_sort_voting_items' ) );
			add_action( 'wp_ajax_upload_item_thumbnail', array( &$this, 'ajax_upload_item_thumbnail' ) );
			
		} //end constructor
		
		public function ajax_approve_voting_item() {
			//Verify the nonce
			$post_id = isset( $_REQUEST['post_id'] ) ? absint( $_REQUEST['post_id'] ) : false;
			if ( !$post_id || !wp_verify_nonce( $_REQUEST['_ajax_nonce'], 'thumbs-up_approve-item-' . $post_id ) || !current_user_can( 'administrator' ) ) die( "-1" );
			
			$post = get_post( $post_id );
			$post->post_status = 'publish';
			$result = wp_update_post( $post );
			die( json_encode( array( 'approve_status' => $result, 'post_id' => $post_id ) ) );
		} //end ajax_approve_voting_item
		
		public function ajax_submit_voting_group() {
			//Privilege check
			if ( !current_user_can( 'administrator' ) || !wp_verify_nonce( $_REQUEST['_ajax_nonce'], 'thumbs-up_add-new' ) ) die( json_encode( array( 'save_error' => 'You do not have the appropriate privileges to perform this action' ) ) );
			
			//Get the form fields
			parse_str( $_POST['ajax_form_data'], $form_data );
			$post_id = isset( $form_data['post'] ) ? absint( $form_data['post'] ) : false;
			$action = sanitize_text_field( $form_data['action'] );
			$title = empty( $form_data['group_title'] ) ? false : apply_filters( 'title_save_pre', sanitize_text_field( $form_data['group_title'] ) );
			$description = empty( $form_data['group_description'] ) ? false : apply_filters( 'content_save_pre', $form_data['group_description'] );
			$media_type = sanitize_text_field( $form_data['media_type'] );
			
			//Get date/time
			$timestamp = $has_expiration = false;
			if ( $form_data['voting_expire'] == 'true' ) {
				if ( !empty( $form_data['date'] ) && !empty( $form_data['time'] ) ) {
					$timestamp = (string)strtotime( $form_data['date'] . ' ' . $form_data['time'] );
					$has_expiration = true;
				}
			} else {
				//Make timestamp 100 years from now
				$timestamp = time() + 31557600;
			}
			
			//Check required fields and spit out any errors
			if ( trim( $title ) == '' ) {
				die( json_encode( array( 'validation_error' => __( 'This field is required', 'it-l10n-thumbsup' ) ) ) );
			}
			
			//Cool, everything is validated - Let's create the post
			$postargs = array(
				'post_title' => $title,
				'post_content' => $description,
				'post_status' => 'publish',
				'post_type' => 'it_voting_group'
			);
			//Set post ID if we're updating the post
			if ( $post_id ) $postargs['ID'] = $post_id;
			
			$post_id = wp_insert_post( $postargs );
			if ( !$post_id ) {
				//Post could not be created, return an error
				die( json_encode( array( 'save_error' => __( 'Could not save the voting group.', 'it-l10n-thumbsup' ) ) ) );
			}
			
			//Add appropriate post meta
			update_post_meta( $post_id, 'expires', $has_expiration );
			update_post_meta( $post_id, 'expiration_timestamp', $timestamp );
			update_post_meta( $post_id, 'media_type', $media_type );
			
			//Yay, all is well, return what we need for JavaScript
			$time = $date = $expiration = false;
			if ( $has_expiration ) {
				$time = date( 'G:i', $timestamp );
				$date = date( 'm/d/Y', $timestamp );
				$expiration = date( get_option('date_format'), $timestamp ) . ' at ' . date( get_option('time_format'), $timestamp );
			} else {
				$expiration = __( "No expiration", 'it-l10n-thumbsup' );
			}
			$post = get_post( $post_id );
			$return = array(
				'action' => esc_js( $action ),
				'post_id' => esc_js( $post_id ),
				'content' => stripslashes( apply_filters( 'pb_the_content', $post->post_content ) ),
				'title' => stripslashes( apply_filters( 'the_title', $post->post_title ) ),
				'expiration_display' => esc_js( $expiration ),
			);
			die( json_encode( $return ) );
		} //end ajax_submit_voting_group
		
		public function ajax_submit_voting_item() {
			//Privilege check
			if ( !current_user_can( 'administrator' ) || !wp_verify_nonce( $_REQUEST['_ajax_nonce'], 'thumbs-up_add-new-item' ) ) die( json_encode( array( 'save_error' => __( 'You do not have the appropriate privileges to perform this action', 'it-l10n-thumbsup' ) ) ) );
			
			//Get the form fields
			parse_str( $_POST['ajax_form_data'], $form_data );
			$post_status = isset( $form_data[ 'post_status' ] ) ? $form_data[ 'post_status' ] : 'publish';
			$post_id = isset( $form_data['item_id'] ) ? absint( $form_data['item_id'] ) : false;
			$parent_post_id = isset( $form_data['parent_id'] ) ? absint( $form_data['parent_id'] ) : false;
			$attachment_id = isset( $form_data['attachment_id'] ) ? absint( $form_data['attachment_id'] ) : false;
			$menu_order = isset( $form_data['menu_order'] ) ? absint( $form_data['menu_order'] ) : 0;
			$action = sanitize_text_field( $form_data['action'] );
			$title = empty( $form_data['item_title'] ) ? false : apply_filters( 'title_save_pre', sanitize_text_field( $form_data['item_title'] ) );
			$item_description = isset( $form_data[ 'item_description' ] ) ? apply_filters( 'content_save_pre', $form_data[ 'item_description' ] ) : false;
			$video_url = isset( $form_data['item_video_url'] ) ? esc_url_raw( sanitize_text_field( $form_data['item_video_url'] ) ) : false;
			$video_embed_html = false;
			$attachment_caption = isset( $form_data['image_caption'] ) ? apply_filters( 'title_save_pre', sanitize_text_field( $form_data['image_caption'] ) ) : false;
			$attachment_description = isset( $form_data['image_description'] ) ? apply_filters( 'content_save_pre', $form_data['image_description'] ) : false;
			$media_type = sanitize_text_field( $form_data['media_type'] );
			
			//Check to see if we're updating
			$updating = false;
			if ( isset( $form_data[ 'updating' ] ) ) {
				$updating = $form_data[ 'updating' ] == "1" ? true : false;
			} //end if isset updating
			
			$errors = array();
			//Check required fields and spit out any errors
			if ( trim( $title ) == '' ) {
				array_push( $errors, array( 'field' => 'item_title', 'message' => __( 'This field is required', 'it-l10n-thumbsup' ) ) );
			}
			if ( $media_type == 'videos' && !$video_url ) {
				array_push( $errors, array( 'field' => 'item_video_url', 'message' => __( 'This field is required','it-l10n-thumbsup' ) ) );
			}
			//Check to see if video can be embedded (if applicable)
			if ( $media_type == 'videos' ) {
				$video_embed_html = wp_oembed_get( $video_url, array( 'width' => 300, 'height' => 300 ) );
				if ( !$video_embed_html ) {
					array_push( $errors, array( 'field' => 'item_video_url', 'message' => __( 'This video can not be embedded.  The owner of the video may have disallowed video embeds.', 'it-l10n-thumbsup' ) ) );
				}
			}
		
			if ( count( $errors ) > 0 ) die( json_encode( array( 'errors' => $errors ) ) );
			
			
			//Cool, everything is validated - Let's create the post
			$postargs = array(
				'post_title' => $title,
				'post_status' => $post_status,
				'post_parent' => $parent_post_id,
				'post_type' => 'it_voting_group_item',
				'menu_order' => $menu_order
			);
			//Set post ID if we're updating the post
			if ( $post_id  ) { 
				$postargs['ID'] = $post_id;
			} //end if $post_id
			
			//Set post_content if saving a video
			if ( $video_url ) $postargs['post_content'] = $video_url;
			
			//Set the post content is saving an item description
			if ( $item_description) $postargs['post_content'] = $item_description;
			
			$post_id = wp_insert_post( $postargs );
			if ( !$post_id ) {
				//Post could not be created, return an error
				die( json_encode( array( 'save_error' => __( 'Could not save the item.', 'it-l10n-thumbsup' ) ) ) );
			}
			
			$post = get_post( $post_id );
			$title = stripslashes( apply_filters( 'the_title', $post->post_title ) );
			
			if ( $media_type == 'images' && $attachment_id ) {
				$attachment_data = wp_get_attachment_image_src( $attachment_id, 'thumbnail' );
				$attachment_image = wp_get_attachment_image( $attachment_id, array( '100', '100' ) );
				
				//Update the attachment
				$postargs = array(
					'ID' => $attachment_id,
					'post_parent' => $post_id,
					'post_title' => $attachment_caption,
					'post_status' => 'inherit',
					'post_content' => $attachment_description,
					'post_type' => 'attachment'
				);
				//todo use wp_insert_attachment instead
				wp_insert_post( $postargs );
			} //end if $media_type == 'images'
			
			//Get HTML
			$html = $this->build_item_view_html( $media_type, $title, $post_id, $attachment_id, $video_embed_html );
					
			//Let's update the rating for the items 
			$this->parent->calculate_ratings( array( 'item_id' => $post_id, 'parent_id' => $parent_post_id ) );
			
			//Now let's return
			$return = array(
				'action' => esc_js( $action ),
				'post_id' => esc_js( $post_id ),
				'title' => $title,
				'html' => $html,
				'menu_order' => $menu_order + 1,
				'post_status' => $post_status
			);
			if ( $updating ) $return[ 'updating' ] = true;
			die( str_replace( '[', '\u0022', json_encode( $return ) ) );
		} //end ajax_submit_voting_item
		//Returns HTML for when viewing the items in the admin panel
		
		
		public function ajax_delete_voting_item() {
			//Verify the nonce
			$post_id = isset( $_REQUEST['post_id'] ) ? absint( $_REQUEST['post_id'] ) : false;
			if ( !$post_id || !wp_verify_nonce( $_REQUEST['_ajax_nonce'], 'thumbs-up_delete-item-' . $post_id ) || !current_user_can( 'administrator' ) ) die( "-1" );
			
			//Delete the post
			$result = wp_delete_post( $post_id ) ? true : false;
			die( json_encode( array( 'deletion_status' => $result, 'post_id' => $post_id ) ) );
		} //end ajax_delete_voting_item
		
		public function ajax_edit_voting_item() {
			//Verify nonce
			$post_id = isset( $_REQUEST['post_id'] ) ? absint( $_REQUEST['post_id'] ) : false;
			if ( !$post_id || !wp_verify_nonce( $_REQUEST['_ajax_nonce'], 'thumbs-up_edit-item-' . $post_id ) || !current_user_can( 'administrator' ) ) die( "-1" );
			
			//Get the item
			$item = get_post( $post_id );
			if ( !$item ) { return json_encode( array( 'error', __( 'Unable to edit item' ) ) ); }
			
			//Get the media type
			$media_type = 	get_post_meta( $item->post_parent, 'media_type', true );
			$return = array();
			
			//Get global data
			$return[ 'item_title' ] = stripslashes( apply_filters( 'title_edit_pre', $item->post_title ) );
			$return[ 'media_type' ] = $media_type;
			$return[ 'item_id' ] = absint( $post_id );
			$return[ 'menu_order' ] = absint( $item->menu_order );
			$return[ 'post_status' ] = esc_js( $item->post_status );
			
			//Get topic data, if applicable
			if ( $media_type == 'topics' ) {
				$return[ 'item_description' ] = stripslashes( apply_filters( 'content_edit_pre', $item->post_content ) );
			} //end $media_type topics
			
			//Get Image Data, if applicable
			if ( $media_type == 'images' ) {
				$attachment_id = get_post_meta( $item->ID, '_thumbnail_id', true );
				if ( $attachment_id ) {
					$attachment = get_post( $attachment_id );
					if ( $attachment ) {
						$attachment_data = wp_get_attachment_image_src( $attachment_id, 'thumbnail' );
						$attachment_image = wp_get_attachment_image( $attachment_id, array( '100', '100' ) );
						$return [ 'attachment_id' ] = $attachment_id;
						$return[ 'voting_item_thumbnail' ] = $attachment_image;
						$return[ 'image_caption' ] = stripslashes( apply_filters( 'title_edit_pre', $attachment->post_title ) );
						$return[ 'image_description' ] = stripslashes( apply_filters( 'content_edit_pre', $attachment->post_content ) );
					} //end if $attachment
				} //end if $attachment_id
			} // end $media_type images
			
			//Get Video Data, if applicable
			if ( $media_type == 'videos' ) {
				$return[ 'item_video_url' ] = stripslashes( trim( apply_filters( 'content_edit_pre', $item->post_content ) ) );
			} //end $media_type videos
			
			//Output
			die( json_encode( $return ) );
			
		} //end function ajax_edit_voting_item
		
		public function ajax_sort_voting_items() {
			//verify privileges
			$post_ids = isset( $_REQUEST['post_ids'] ) ? $_REQUEST['post_ids'] : false;
			if ( !current_user_can( 'administrator' ) || !$post_ids ) die( "-1" );
			
			$items = array();
			if ( preg_match_all( '/(\d+)/', $post_ids, $matches ) )
				$items = $matches[1];
			
			$counter = 0;
			foreach ( $items as $item_id ) {
				$args = array( 
					'ID' => absint( $item_id ),
					'menu_order' => $counter
				);
				wp_update_post( $args );
				$counter += 1;
			} //end foreach
			die ( json_encode( array( 'menu_order' => count( $items ) ) ) );
		} //end ajax_sort_voting_items
		
		public function ajax_upload_item_thumbnail() {
			//Get data and validate
			$parent_post_id = isset( $_REQUEST['item_post_parent'] ) ? absint( $_REQUEST['item_post_parent'] ) : false;
			if ( !$parent_post_id || !wp_verify_nonce( $_REQUEST['_ajax_nonce'], 'pb_upload_item_thumb' ) || !current_user_can( 'administrator' ) || empty( $_FILES ) ) die( json_encode( array( 'error' => __( 'Could not upload image', 'it-l10n-thumbsup' ) ) ) );
						
			//Check type
			switch( $_FILES['thumbnail']['type'] ) {
				case "image/x-png":
				case "image/png":
				case "image/jpg":
				case "image/jpeg":
				case "image/pjpeg":
				case "image/gif":
					break;
				default:
					die( json_encode( array( 'error' => __( 'Invalid image type', 'it-l10n-thumbsup' ) . $_FILES['thumbnail']['type'] ) ) );
					break;
			} //end switch
			
			
			$menu_order = isset( $_REQUEST['menu_order'] ) ? absint( $_REQUEST['menu_order'] ) : 0;
			
			$title = __( 'Temporary Title', 'it-l10n-thumbsup' );
			//Cool, everything is validated - Let's create the item post
			$postargs = array(
				'post_title' => $title,
				'post_status' => 'draft',
				'post_parent' => $parent_post_id,
				'post_type' => 'it_voting_group_item',
				'menu_order' => $menu_order
			);
			$item_id = wp_insert_post( $postargs );
			if ( !$item_id ) die( json_encode( array( 'error' => __( 'Could not upload image', 'it-l10n-thumbsup' ) ) ) );
			
			//Let's read in the file params and attach
			$attachment_id = media_handle_upload( 'thumbnail' , $item_id );
			if ( !$attachment_id ) { 
				wp_delete_post( $item_id, true );
				die( json_encode( array( 'error' => __( 'Could not upload image', 'it-l10n-thumbsup' ) ) ) );
			}
			update_post_meta($item_id,'_thumbnail_id',$attachment_id);
			
			$attachment_data = wp_get_attachment_image_src( $attachment_id, array( '100', '100' ) );

			//\u0022
			$return = array(
				'title' => $title,
				'item_id' => $item_id,
				'attachment_id' => $attachment_id,
				'image' => $attachment_data
			);
			die( json_encode( $return ) );
		} //end ajax_upload_item_thumbnail
		
		private function alert() {
			$args = func_get_args();
			return call_user_func_array( array( $this->parent, 'alert' ), $args );
		}
		
		private function video() {
			$args = func_get_args();
			return call_user_func_array( array( $this->parent, 'video' ), $args );
		}
		
		private function tip() {
			$args = func_get_args();
			return call_user_func_array( array( $this->parent, 'tip' ), $args );
		}
		
		private function log() {
			$args = func_get_args();
			return call_user_func_array( array( $this->parent, 'log' ), $args );
		}
		
		private function get_plugin_info() {
			$args = func_get_args();
			return call_user_func_array( array( $this->parent, 'get_plugin_info' ), $args );
		}
		
		private function get_plugin_url() {
			$args = func_get_args();
			return call_user_func_array( array( $this->parent, 'get_plugin_url' ), $args );
		}
		private function get_plugin_dir() {
			$args = func_get_args();
			return call_user_func_array( array( $this->parent, 'get_plugin_dir' ), $args );
		}
		
		
		private function title( $title ) {
			echo '<h2><img src="' . $this->get_plugin_url( '/images/icon.png' ) . '" style="vertical-align: -7px;"> ' . $title . '</h2>';
		}
		
		
		public function admin_scripts() {
			wp_enqueue_script( 'pluginbuddy-tooltip-js', $this->get_plugin_url( '/js/tooltip.js' ), array( 'jquery', 'dashboard' ), $this->get_plugin_info( 'version' ), true );
			wp_enqueue_script( 'pluginbuddy-'. $this->get_plugin_info( 'slug' ) .'-admin-js', $this->get_plugin_url( '/js/admin.js' ), array( 'jquery', 'dashboard', 'jquery-ui-sortable', 'jquery-form' ), $this->get_plugin_info( 'version' ), true );
				wp_localize_script( 'pluginbuddy-' . $this->get_plugin_info( 'slug' ). '-admin-js', 'pb_thumbsup',
					array( 'saving' => __( 'Saving...', 'it-l10n-thumbsup' ),
						'add_voting_item' => __( 'Add Voting Item', 'it-l10n-thumbsup' ),
						'edit_voting_item' => __( 'Edit Voting Item', 'it-l10n-thumbsup' ),
						'save' => __( 'Save' , 'it-l10n-thumbsup' ),
						'add_new_item' => __( 'Add New Item', 'it-l10n-thumbsup' ),
						'uploading' => __( 'Uploading...', 'it-l10n-thumbsup' ),
						'upload_image' => __( 'Upload Image', 'it-l10n-thumbsup' ),
						'delete_item_error' => __( 'Could not delete item', 'it-l10n-thumbsup' ),
						'edit_item_error' => __( 'Could not edit item', 'it-l10n-thumbsup' )
					) 
				);
			
			//todo - figure out a better way to selectively load these scripts
			wp_enqueue_script( 'jquery-ui-timepicker' );
			wp_enqueue_script( 'jquery-ui-datepicker' );
		} //end admin_scripts
		
		public function admin_styles() {
			wp_enqueue_style( 'pluginbuddy-'. $this->get_plugin_info( 'slug' ) .'-admin-css', $this->get_plugin_url( '/css/admin.css' ) );
			print_r('test');
			//todo - figure out a better way to selectively load these styles
			wp_enqueue_style( 'jquery-ui-timepicker' );
			wp_enqueue_style( 'jquery-ui-datepicker' );
		} //end admin_styles
		
		public function build_item_view_html( $media_type = 'topics', $title, $item_id = false, $attachment_id = false,$video_embed_html = false ) {
			//Get the post status
			$item = get_post( $item_id );
			$post_status = $item->post_status;
			//Build the HTML
			$html = sprintf( "<li id='voting_item_%d' class='%s'>", absint( $item_id ), esc_attr( $post_status ) );
			$html .= "<h3>{$title}</h3>";
			
			//Show video, if applicable
			if ( $media_type == 'videos' && $video_embed_html ) {
				$html .= sprintf( "<div class='voting_item_video'>%s</div>", $video_embed_html );
			} //end if $video_embed_html
			
			//Show image, if applicable
			if ( $media_type == 'images' && $attachment_id ) {
				$attachment_data = wp_get_attachment_image_src( $attachment_id, 'thumbnail' );
				$attachment_image = wp_get_attachment_image( $attachment_id, array( '100', '100' ) );
				$html .= "<div class='voting_item_thumbnail'>{$attachment_image}</div>";
			} //end $media_type images
			
			//Show Edit | Delete buttons
			$html .= "<div class='voting_item_title'>";
			//Show approve button if post is in moderation
            	if ( $post_status == 'draft' ) {
		       	$html .= sprintf( "<span id='thumbsup_approve_%d'><a class='button-primary' href='%s' onclick='return jQuery.pb_thumbsup.approve_item( %d, \"%s\" );'>Approve</a>", $item_id, esc_url( add_query_arg( array( 'pid' => $item_id ), admin_url( 'admin-ajax.php' ) ) ), $item_id, wp_create_nonce( 'thumbs-up_approve-item-' . $item_id ) );
		       	$html .= "&nbsp;|&nbsp;</span>";
		      }
            	$html .= sprintf( "<a class='button-secondary' href='%s' onclick='return jQuery.pb_thumbsup.edit_item( %d, \"%s\" );'>Edit</a>", esc_url( add_query_arg( array( 'pid' => $item_id ), admin_url( 'admin-ajax.php' ) ) ), $item_id, wp_create_nonce( 'thumbs-up_edit-item-' . $item_id ) );
            	$html .= "&nbsp;|&nbsp;";
            	$html .= sprintf( "<a class='button-secondary' href='%s' onclick='return jQuery.pb_thumbsup.delete_item( %d, \"%s\" );'>Delete</a>", esc_url( add_query_arg( array( 'pid' => $item_id ), admin_url( 'admin-ajax.php' ) ) ), $item_id, wp_create_nonce( 'thumbs-up_delete-item-' . $item_id ) );
            	

            	
            	
            	
            	$html .= "</div><!--/.voting_item_title-->";
            	$html .= "</li><!--/voting_item-->";
            	
            	
            	
            	return $html;

		} //end build_item_view_html
		
		/**
		 *	get_feed()
		 *
		 *	Gets an RSS or other feed and inserts it as a list of links...
		 *
		 *	$feed		string		URL to the feed.
		 *	$limit		integer		Number of items to retrieve.
		 *	$append		string		HTML to include in the list. Should usually be <li> items including the <li> code.
		 *	$replace	string		String to replace in every title returned. ie twitter includes your own username at the beginning of each line.
		 *	$cache_time	int			Amount of time to cache the feed, in seconds.
		 */
		private function get_feed( $feed, $limit, $append = '', $replace = '', $cache_time = 300 ) {
			require_once(ABSPATH.WPINC.'/feed.php');  
			$rss = fetch_feed( $feed );
			if (!is_wp_error( $rss ) ) {
				$maxitems = $rss->get_item_quantity( $limit ); // Limit 
				$rss_items = $rss->get_items(0, $maxitems); 
				
				echo '<ul class="pluginbuddy-nodecor">';

				$feed_html = get_transient( md5( $feed ) );
				if ( $feed_html == '' ) {
					foreach ( (array) $rss_items as $item ) {
						$feed_html .= '<li>- <a href="' . $item->get_permalink() . '">';
						$title =  $item->get_title(); //, ENT_NOQUOTES, 'UTF-8');
						if ( $replace != '' ) {
							$title = str_replace( $replace, '', $title );
						}
						if ( strlen( $title ) < 30 ) {
							$feed_html .= $title;
						} else {
							$feed_html .= substr( $title, 0, 32 ) . ' ...';
						}
						$feed_html .= '</a></li>';
					}
					set_transient( md5( $feed ), $feed_html, $cache_time ); // expires in 300secs aka 5min
				}
				echo $feed_html;
				
				echo $append;
				echo '</ul>';
			} else {
				echo 'Temporarily unable to load feed...';
			}
		}
		
		
		public function view_gettingstarted() {
			require( 'view_gettingstarted.php' );
		}
		
		public function view_post_settings() {
			require( 'view_post_settings.php' );
		} //end view_post_settings
		public function view_settings() {
			require( 'view_settings.php' );
		}
		public function view_add_new() {
			require( 'view_addnew.php' );
		}
		public function view_edit_items() {
			require( 'view_edit.php' );
		}
		public function view_stats() {
			require( 'view_stats.php' );
		}
		public function view_post_stats() {
			require( 'view_post_stats.php' );
		}
		
		/** admin_menu()
		 *
		 * Initialize menu for admin section.
		 *
		 */		
		public function admin_menu() {
			$dependencies = array();
			// Add main menu (default when clicking top of menu)
			add_menu_page( $this->get_plugin_info( 'name' ) . __( ' Getting Started', 'it-l10n-thumbsup' ), $this->get_plugin_info( 'name' ), 'administrator', $this->get_plugin_info( 'slug' ), array(&$this, 'view_gettingstarted'), $this->get_plugin_url( '/images/thumbsup16.png' ) );
			//Show 
			$dependencies[] = add_submenu_page( $this->get_plugin_info( 'slug' ),  $this->get_plugin_info( 'name' ) . __( ' Getting Started', 'it-l10n-thumbsup' ), __( 'Getting Started', 'it-l10n-thumbsup' ), 'administrator',  $this->get_plugin_info( 'slug' ), array(&$this, 'view_gettingstarted'));
			//Load menu items and their callbacks
			foreach ( $this->menu as $slug => &$args ) {
				$args['pagehook'] = add_submenu_page( $this->get_plugin_info( 'slug' ), $this->get_plugin_info( 'name' ) . ' ' . $args[ 'title' ], $args[ 'title' ], 'administrator', sprintf( '%s-%s', $this->get_plugin_info( 'slug' ), $slug ), array(&$this, $args[ 'callback' ] ) );
				$dependencies[] = $args['pagehook'];
			} //end foreach
			//Queue scripts/styles for dependencies
			foreach ( $dependencies as $pagehook ) {
				add_action( 'admin_print_scripts-' .$pagehook, array( &$this, 'admin_scripts' ) );
				add_action( 'admin_print_styles-' .$pagehook, array( &$this, 'admin_styles' ) );
			} //end foreach
				
		} //end admin_menu
		
	} // End class
} //end if class exists
$pluginbuddy_thumbsup_admin = new pluginbuddy_thumbsup_admin( $this );
