<?php
class pluginbuddy_posttype_display {
	private $posts = false;
	private $posts_per_page = 10;
	private $post_type = false;
	private $columns = false;
	
	function __construct( $args ) {
		$defaults = array(
			'post_type' => false,
		);
		extract( wp_parse_args( $args, $defaults ) );
		$this->post_type = $post_type;
		
		$columns = array(
			'post_title' => 'Title',
			'post_content' => 'Content',
			'post_date' => 'Date',
		);
		$this->columns = apply_filters( 'pb_posttype_columns', $columns );
	} //end constructor
	public function display() {
		global $paged;
		$offset = $paged * $this->posts_per_page;
		//todo - implement paging on next version if necessary
		$args = array(
			'numberposts' => -1,
			'posts_per_page' => -1/*$this->posts_per_page*/,
			'post_type' => $this->post_type,
			'post_status' => 'publish',
			'offset' => $offset
		);
		$args = apply_filters( 'pb_posttype_args', $args );
		$this->posts = get_posts ( $args );
		if ( empty( $this->posts ) ) {
			echo "<p>There are no items to display.</p>";
			return;
		}
		$this->display_column_header();
		foreach ( $this->posts as $post ) {
			$this->display_single_row( $post );
		}
		$this->display_column_footer();
	} //end display
	private function display_column_header() {
	?>
    <table cellspacing="0" class="wp-list-table widefat fixed posts">
	<thead>
	<tr>
    <?php
		foreach ( $this->columns as $type => $title ) {
			?>
            <th style="" class="manage-column column-<?php echo esc_attr( $type ); ?>" scope="col"><?php echo esc_html( $title ); ?></th>
            <?php
		} //end foreach
	?>
		
    </tr>
    </thead>
    <tbody id='the-list'>
    <?php
	} //end display_column_header
	private function display_column_footer() {
		?>
        </tbody>
        <tfoot>
	<tr>
    <?php
		foreach ( $this->columns as $type => $title ) {
			?>
            <th style="" class="manage-column column-<?php echo esc_attr( $type ); ?>" scope="col"><?php echo esc_html( $title ); ?></th>
            <?php
		} //end foreach
	?>
		
    </tr>
    </tfoot>
    </table>
        <?php
	
	} //end display_column_footer
	private function display_single_column( $column_type, $post ) {
		$column_data = false;
		//todo - clean these up a bit
		switch( $column_type ) {
			case 'post_title':
				$column_data = apply_filters( 'the_title', $post->post_title );
				break;
			case 'post_content':
				$column_data = stripslashes( apply_filters( 'pb_the_content', $post->post_content ) );
				break;
			case 'post_date':
				$timestamp = strtotime( $post->post_date );
				$column_data = date( get_option('date_format'), $timestamp ) . ' at ' . date( get_option('time_format'), $timestamp );
				break;
		}
		$column_data = apply_filters( 'pb_posttype_column', $column_data, $column_type, $post );
		echo $column_data;
	} //end display_single_column
	private function display_single_row( $post ) {
		?>
        <tr>
        <?php
		foreach ( $this->columns as $type => $title ) {
			?>
            <td><?php $this->display_single_column( $type, $post ); ?></td>
            <?php
		}
		?>
        </tr>
        <?php
	} //end display_single_row
} //end class