<?php
/*
Plugin Name: Thumbs Up
Plugin URI: http://pluginbuddy.com/purchase/thumbsup/
Description: Allows you to rank items based on a thumbs up or thumbs down
Author: The PluginBuddy Team
Version: 1.3.9
Requires at least: 3.0
Author URI: http://www.pluginbuddy.com/
*/ 
if ( !function_exists( 'wp_print_r' ) ) {
	function wp_print_r( $args, $die = true ) {
		$echo = '<pre>' . print_r( $args, true ) . '</pre>';
		if ( $die ) die( $echo );
		else echo $echo;
	}
}
if (!class_exists('pluginbuddy_thumbsup')) {
    class pluginbuddy_thumbsup	{	
		//public
		public $upgrade = false;
		
		//private
		private $admin_options = array();
		private $errors = '';
		private $plugin_url = '';
		private $plugin_dir = '';
		private $plugin_info = array();
		private $user_hash = false;
		private $table_version = '1.2.0';
		private $tablename = 'wp_thumbsup_stats';
		private $_updater = '1.0.8';
		
		/**
		* __construct()
		* 
		* Class constructor
		*
		*/
		function __construct(){
			
			//Initialize plugin information
			$this->plugin_info = array(
				'slug' => 'pluginbuddy_thumbsup',
				'version' => '1.3.9',
				'name' => 'Thumbs Up',
				'url' => 'http://pluginbuddy.com/purchase/thumbsup/',
				'locale' => 'LION',
				'path' => plugin_basename( __FILE__ )
			);
			
			$this->admin_options = $this->get_admin_options();
			$this->plugin_url = rtrim( plugin_dir_url(__FILE__), '/' );
			$this->plugin_dir = rtrim( plugin_dir_path(__FILE__), '/' );
			
			//Variables for iThemes integration
			$this->_defaults = array();
			$this->_version = $this->plugin_info[ 'version' ];
			$this->_var = $this->plugin_info[ 'slug' ];
			$this->_pluginURL = $this->plugin_url;
			$this->_pluginPath = $this->plugin_dir;
			$this->_pluginBase = plugin_basename( __FILE__  );
			
			add_action( 'init', array( &$this, 'init' ) );
			add_action( 'init', array( &$this, 'register_thumb_css' ) );
			add_action( 'init', array( &$this, 'register_thumb_script' ) );
			
			//Register our scripts and styles
			
			

			
		} //end constructor
		
		function register_thumb_css() { 
			wp_register_style( 'pb_thumbsup_css', $this->get_plugin_url( 'css/thumbsup.css' ), array(), $this->get_plugin_info( 'version' ), 'all' );
		}
		
		function register_thumb_script() { 
			wp_register_script( 'pb_thumbsup_js', $this->get_plugin_url( 'js/thumbsup.js' ), array( 'jquery' ), $this->get_plugin_info( 'version' ), true );
		}
		
		/**
		* add_footer_html()
		* 
		* Adds a loading image into the footer that all widgets and shortcodes can use
		*
		*/
		public function add_footer_html() {
			$html = sprintf( "<div id='pb_thumbsup_loading' style='display: none;'><img id='pb_thumbsup_loading_gif' src='%s' alt='%s' /></div> ", esc_url( $this->get_plugin_url( 'images/ajax-loader.gif' ) ), __( 'Show Image Description', 'it-l10n-thumbsup' ) );
			echo $html;
		} //end add_footer_html
		
		/**
		* add_media_button()
		* 
		* Displays a thumbsup icon in the media bar when editing a post or page
		*
		* @param		string    $context	Media bar string
		* @return		string    Updated context variable with shortcode button added
		*/
		public function add_media_button( $context ) {
			$image_btn = $this->get_plugin_url( 'images/thumb_shortcode.png' );
			$out = '<a href="#TB_inline?width=450&height=600&inlineId=pb_thumbsup_shortcode" class="thickbox" title="' . __("Add ThumbsUp Shortcode", 'it-l10n-thumbsup' ) . '"><img src="'.$image_btn.'" alt="' . __("Add ThumbsUp Shortcode", 'it-l10n-thumbsup' ) . '" /></a>';
			return $context . $out;
		} //end add_media_button
		
		public function add_post_scripts( $skip = false ) {
			global $post, $wp_query;
			if ( !$skip ) {
				if ( !is_object( $post ) ) return;
				if ( !in_array( get_post_type(), $this->get_admin_option( 'post_types' ) ) ) return;
			}

			wp_enqueue_script( 'pb_thumbsup_js' );
			wp_localize_script( 'pb_thumbsup_js', 'thumbsup', array( 
				'post_display' => esc_js( $this->get_admin_option( 'post_voting_display' ) ),
				'person_up' => __( 'person likes this.', 'it-l10n-thumbsup' ),
				'people_up' => __( 'people like this.', 'it-l10n-thumbsup' ),
				'person_down' => __( 'person does not like this.', 'it-l10n-thumbsup' ),
				'people_down' => __( 'people do not like this.', 'it-l10n-thumbsup' ),
				'saving' => __( 'Saving...', 'it-l10n-thumbsup' ),
				'suggest_thanks' => __( 'Thanks for the suggestion', 'it-l10n-thumbsup' )
			) ); 
			
		} //end add_post_scripts
		
		public function add_post_styles() {
			global $post;
			if ( !is_object( $post ) ) return;
			if ( !in_array( get_post_type(), $this->get_admin_option( 'post_types' ) ) ) return;
			
			wp_enqueue_style( 'pb_thumbsup_css' );
		
		} //end add_post_scripts
		
		/**
		* add_thumbsup_shortcode()
		* 
		* Action target that displays the popup to insert the thumbsup shortcode on a post or page
		*
		*/
		public function add_thumbsup_shortcode(){
			?>
			<script type="text/javascript">
				jQuery(document).ready(function() {
					jQuery( "select#pb_add_thumbsup_shortcode" ).bind( "change", function() {
						var $selected = jQuery( "select#pb_add_thumbsup_shortcode option:selected" );
						if ( $selected.val() == '' ) { return; }
						var media_type = $selected.attr( 'rel' );
						//Open or close video/image div areas
						if ( media_type == 'videos' ) {
							jQuery( "#pb_thumbsup_images, #pb_thumbsup_desc" ).hide();
							jQuery( "#pb_thumbsup_videos, #pb_thumbsup_title" ).show();
						} else if ( media_type == 'images' ) {
							jQuery( "#pb_thumbsup_images, #pb_thumbsup_title" ).show();
							jQuery( "#pb_thumbsup_videos, #pb_thumbsup_desc" ).hide();
						} else {
							jQuery( "#pb_thumbsup_images, #pb_thumbsup_videos, #pb_thumbsup_title" ).hide();
							jQuery( "#pb_thumbsup_desc" ).show();
						}	
					} );
				} );
				function pb_InsertGroup(){
					var $selected = jQuery( "select#pb_add_thumbsup_shortcode option:selected" );
					if ( $selected.val() == '' ) { return; }
					
					//Get ID and Media Type
					var attributes = '';
					var group_id = $selected.val();
					var media_type = $selected.attr( 'rel' );
					attributes += ' group_id="' + group_id + '" ';
					
					//Get Display params
					var display = jQuery( "select[name='pb_thumbsup_display'] option:selected" ).val();
					attributes += 'display="' + display + '" ';
					
					//Get sort params
					var orderby = jQuery( "select[name='pb_thumbsup_sort'] option:selected" ).val();
					var order = jQuery( "select[name='pb_thumbsup_sortorder'] option:selected" ).val();
					attributes += 'orderby="' + orderby + '" ';
					attributes += 'order="' + order + '" ';
					
					//Get group params
					var show_group_title = jQuery( "input[name='pb_thumbsup_group_title']:checked" ).val();
					var show_group_desc = jQuery( "input[name='pb_thumbsup_group_desc']:checked" ).val();
					attributes += 'show_group_title="' + show_group_title + '" ';
					attributes += 'show_group_desc="' + show_group_desc + '" ';
					
					//Get global item params
					if ( media_type != 'topics' ) {
						var show_item_title = jQuery( "input[name='pb_thumbsup_item_title']:checked" ).val();
						attributes += 'show_item_title="' + show_item_title + '" ';
					} else {
						var show_item_desc = jQuery( "input[name='pb_thumbsup_item_desc']:checked" ).val();
						attributes += 'show_item_desc="' + show_item_desc + '" ';
						attributes += 'show_item_title="1" '; //Topics must always show the title
					}
					
					//Get Image params
					if ( media_type == 'images' ) {
						var max_height = jQuery( "input[name='pb_image_max_height']").val();
						if ( max_height != '' ) {
							attributes += 'thumb_height="' + max_height + '" ';
						}
						var max_width = jQuery( "input[name='pb_image_max_width']").val();
						if ( max_width != '' ) {
							attributes += 'thumb_width="' + max_width + '" ';
						}
						var show_image_caption = jQuery( "input[name='pb_image_caption']:checked" ).val();
						var show_image_desc = jQuery( "input[name='pb_image_desc']:checked" ).val();
						attributes += 'show_image_caption="' + show_image_caption + '" ';
						attributes += 'show_image_desc="' + show_image_desc + '" ';
					}
					//Get Video params
					if ( media_type == 'videos' ) {
						var video_max_height = jQuery( "input[name='pb_video_max_height']").val();
						if ( video_max_height != '' ) {
							attributes += 'video_max_height="' + video_max_height + '" ';
						}
						var video_max_width = jQuery( "input[name='pb_video_max_width']").val();
						if ( video_max_width != '' ) {
							attributes += 'video_max_width="' + video_max_width + '" ';
						}
					}
							
					var win = window.dialogArguments || opener || parent || top;
					win.send_to_editor("[thumbsup " + attributes + "]");
				} //end pb_insertgroup
			</script>
		
			<div id="pb_thumbsup_shortcode" style="display:none;">
				<div class="wrap">
					<div>
						<div style="padding:15px 15px 0 15px;">
							<h3 style="color:#5A5A5A!important; font-family:Georgia,Times New Roman,Times,serif!important; font-size:1.8em!important; font-weight:normal!important;"><?php _e('Insert ThumbsUp Shortcode', 'it-l10n-thumbsup' ); ?></h3>
							<span>
								<?php _e( 'Select an item below.', 'it-l10n-thumbsup' ); ?>
							</span>
						</div>
						<div style="padding:15px 15px 0 15px;">
						<?php
						//Get voting groups
						$group_args = array( 
							'numberposts' => -1,
							'posts_per_page' => -1, 
							'post_type' => 'it_voting_group',
							'order' => 'ASC',
							'orderby' => 'title',
							'meta_key' => 'expiration_timestamp',
							'meta_value' => time(),
							'meta_compare' => '>'
						);
						$voting_groups = get_posts( $group_args );
						if ( !$voting_groups || count( $voting_groups ) == 0 ) :
							?>
                            <p><?php _e( 'There is nothing to add yet.', 'it-l10n-thumbsup' ); ?></p>
                            <?php
						else:
						?>
							<select id="pb_add_thumbsup_shortcode">
								<option value="">  <?php _e( 'Select a Voting Group to Add', 'it-l10n-thumbsup' ); ?>  </option>
								<?php
									foreach($voting_groups as $group) {
										?>
										<option  rel="<?php echo esc_attr( get_post_meta( $group->ID, 'media_type', 1 ) ); ?>" value="<?php echo absint( $group->ID ) ?>"><?php echo esc_html( stripslashes( apply_filters( 'get_the_title', $group->post_title ) ) ) ?></option>
										<?php
									} //end foreach groups
								?>
							</select> <br/>
						</div>

 						<!--Global Options-->
                        <div id='pb_thumbsup_options'>
                        	<table class="form-table">
                                <tbody>
                                	<tr valign="top">
                                        <th scope="row"><label for='pb_thumbsup_display'><?php _e( 'Voting Display', 'it-l10n-thumbsup' ); ?></label></th>
                                        <td>
                                        <select name='pb_thumbsup_display' id='pb_thumbsup_display'>
                                        		<option value='both'><?php _e( 'Thumbs Up and Thumbs Down', 'it-l10n-thumbsup' ); ?></option>
                                        		<option value='up'><?php _e( 'Thumbs Up Only', 'it-l10n-thumbsup' ); ?></option>
                                                <option value='down'><?php _e( 'Thumbs Down Only', 'it-l10n-thumbsup' ); ?></option>
                                        </select> 
                                        </td>
                                    </tr>
                                    <tr valign="top">
                                        <th scope="row"><label for='pb_thumbsup_sort'><?php _e( 'Sort By', 'it-l10n-thumbsup' ); ?></label></th>
                                        <td>
                                        <select name='pb_thumbsup_sort' id='pb_thumbsup_sort'>
                                        		<option value='date'><?php _e( 'Date Added', 'it-l10n-thumbsup' ); ?></option>
                                        		<option value='menu_order'><?php _e( 'Menu Order', 'it-l10n-thumbsup' ); ?></option>
                                                <option value='rand'><?php _e( 'Random', 'it-l10n-thumbsup' ); ?></option>
                                                <option value='rating'><?php _e( 'Rating', 'it-l10n-thumbsup' ); ?></option>
                                        </select> 
                                        <select name='pb_thumbsup_sortorder'>
                                        		<option value='ASC'><?php _e( 'Low to High (ASC)', 'it-l10n-thumbsup' ); ?></option>
                                        		<option value='DESC'><?php _e( 'High to Low (DESC)', 'it-l10n-thumbsup' ); ?></option>
                                        </select>
                                        
                                       
                                        </td>
                                    </tr>
                                    <tr valign="top">
                                        <th scope="row"><?php _e( 'Show Group Title', 'it-l10n-thumbsup' ); ?></th>
                                        <td>
                                        <label for='pb_thumbsup_group_title_yes'><?php _e( 'Yes', 'it-l10n-thumbsup' ); ?></label>
                                        <input type='radio' name='pb_thumbsup_group_title'  id='pb_thumbsup_group_title_yes' value='1'  checked="checked" />&nbsp;<label for='pb_thumbsup_group_title_no'><?php _e( 'No', 'it-l10n-thumbsup' ); ?></label><input type='radio' name='pb_thumbsup_group_title' id='pb_thumbsup_group_title_no' value='0' />
                                        </td>
                                    </tr>
                                    <tr valign="top">
                                        <th scope="row"><?php _e( 'Show Group Description', 'it-l10n-thumbsup' ); ?></th>
                                        <td>
                                        <label for='pb_thumbsup_group_desc_yes'><?php _e( 'Yes', 'it-l10n-thumbsup' ); ?></label>
                                        <input type='radio' name='pb_thumbsup_group_desc'  id='pb_thumbsup_group_desc_yes' value='1'  />&nbsp;<label for='pb_thumbsup_group_desc_no'><?php _e( 'No', 'it-l10n-thumbsup' ); ?></label><input type='radio' name='pb_thumbsup_group_desc' checked="checked"  id='pb_thumbsup_group_desc_no' value='0' />
                                        </td>
                                    </tr>
                                    <tr valign="top" id='pb_thumbsup_title' style='display: none;'>
                                        <th scope="row"><?php _e( 'Show Item Title', 'it-l10n-thumbsup' ); ?></th>
                                        <td>
                                        <label for='pb_thumbsup_item_title_yes'><?php _e( 'Yes', 'it-l10n-thumbsup' ); ?></label>
                                        <input type='radio' name='pb_thumbsup_item_title'  id='pb_thumbsup_item_title_yes' value='1'  checked="checked" />&nbsp;<label for='pb_thumbsup_item_title_no'><?php _e( 'No', 'it-l10n-thumbsup' ); ?></label><input type='radio' name='pb_thumbsup_item_title'   id='pb_thumbsup_item_title_no' value='0' />
                                        </td>
                                    </tr>
                                    <tr valign="top" id='pb_thumbsup_desc' style='display: none;'>
                                        <th scope="row"><?php _e( 'Show Item Description', 'it-l10n-thumbsup' ); ?></th>
                                        <td>
                                        <label for='pb_thumbsup_item_desc_yes'><?php _e( 'Yes', 'it-l10n-thumbsup' ); ?></label>
                                        <input type='radio' name='pb_thumbsup_item_desc'  id='pb_thumbsup_item_desc_yes' value='1'  checked="checked" />&nbsp;<label for='pb_thumbsup_item_desc_no'><?php _e( 'No', 'it-l10n-thumbsup' ); ?></label><input type='radio' name='pb_thumbsup_item_desc'   id='pb_thumbsup_item_desc_no' value='0' />
                                        </td>
                                    </tr>
                        
                                </tbody>
                            </table>
                        </div>
                        
                        <!--Videos-->
                        <div id='pb_thumbsup_videos' style="display: none;">
                        	<table class="form-table">
                                <tbody>
                                    <tr valign="top">
                                        <th scope="row"><label for='pb_video_max_width'><?php _e( 'Video Max Width', 'it-l10n-thumbsup' ); ?></label></th>
                                        <td>
                                        <input type='text' name='pb_video_max_width'  size='10' id='pb_video_max_width' value='' /> 
                                        </td>
                                    </tr>
                                    <tr valign="top">
                                        <th scope="row"><label for='pb_video_max_height'><?php _e( 'Video Max Height', 'it-l10n-thumbsup' ); ?></label></th>
                                        <td>
                                        <input type='text' name='pb_video_max_height'  size='10' id='pb_video_max_height' value='' /> 
                                        </td>
                                    </tr>
                        
                                </tbody>
                            </table>
                        
                        </div><!--/#pb_thumbsup_videos-->
                        <!--/Videos-->
                        <!--Images-->
                        <div id='pb_thumbsup_images' style="display: none;">
                        	<table class="form-table">
                                <tbody>
                                    <tr valign="top">
                                        <th scope="row"><label for='pb_image_max_width'><?php _e( 'Image Max Width', 'it-l10n-thumbsup' ); ?></label></th>
                                        <td>
                                        <input type='text' name='pb_image_max_width'  size='10' id='pb_image_max_width' value='' /> 
                                        </td>
                                    </tr>
                                    <tr valign="top">
                                        <th scope="row"><label for='pb_image_max_height'><?php _e( 'Image Max Height', 'it-l10n-thumbsup' ); ?></label></th>
                                        <td>
                                        <input type='text' name='pb_image_max_height'  size='10' id='pb_image_max_height' value='' /> 
                                        </td>
                                    </tr>
                        			<tr valign="top">
                                        <th scope="row"><?php _e( 'Show Image Caption', 'it-l10n-thumbsup' ); ?></th>
                                        <td>
                                        <label for='pb_image_caption_yes'><?php _e( 'Yes', 'it-l10n-thumbsup' ); ?></label>
                                        <input checked="checked"  type='radio' name='pb_image_caption'  id='pb_image_caption_yes' value='1'  />&nbsp;<label for='pb_image_caption_no'><?php _e( 'No', 'it-l10n-thumbsup' ); ?></label><input type='radio' name='pb_image_caption'  id='pb_image_caption_no' value='0' />
                                        </td>
                                    </tr>
                                    <tr valign="top">
                                        <th scope="row"><?php _e( 'Show Image Description', 'it-l10n-thumbsup' ); ?></th>
                                        <td>
                                        <label for='pb_image_desc_yes'><?php _e( 'Yes', 'it-l10n-thumbsup' ); ?></label>
                                        <input type='radio' name='pb_image_desc'  id='pb_image_desc_yes' value='1'   />&nbsp;<label for='pb_image_desc_no'>No</label><input type='radio' checked="checked" name='pb_image_desc'   id='pb_image_desc_no' value='0' />
                                        </td>
                                    </tr>
                                </tbody>
                            </table>
                        
                        </div><!--/#pb_thumbsup_videos-->
                        <!--/Images-->
                        <?php endif; ?>
						<div style="padding:15px;">
							<input type="button" class="button-primary" value="<?php _e( 'Insert', 'it-l10n-thumbsup' ); ?>" onclick="pb_InsertGroup();"/>&nbsp;&nbsp;&nbsp;
						<a class="button" href="#" onclick="tb_remove(); return false;"><?php _e("Cancel", 'it-l10n-thumbsup' ); ?></a>
						</div>
					</div>
				</div>
			</div>
			<?php
		} //end add_thumbsup_shortcode
		
		private function ajax_post_vote( $item, $vote ) {
			global $current_blog, $wpdb;
			$site_id = $blog_id = 1;
			if ( is_multisite() ) {
				$site_id = $current_blog->site_id;
				$blog_id = $current_blog->blog_id;
			}
			
			$post_id = $item->ID;
			$post_type = get_post_type( $post_id );
			
			//Get the existing rating
			$post_rating = $this->get_post_stats( $post_id );			
			if ( !$post_rating ) {
				//Insert new row because it doesn't exist
				$wpdb->insert( $this->tablename, array( 'site_id' => $site_id, 'blog_id' => $blog_id, 'content_id' => $post_id, 'post_type' => $post_type), array( '%d', '%d', '%d', '%s' ) );
				$post_rating = new stdClass;
				$post_rating->up_votes = 0;
				$post_rating->down_votes = 0;
				$post_rating->total_votes = 0;
			}
			
			//Get the up/down votes
			//Get a transient (if it exists) to determine previous voting behavior
			$one_week = 604800;
			$user_voting_record = get_transient( $this->get_unique_hash( $site_id . $blog_id, $post_id ) );
			if ( !$user_voting_record || $user_voting_record != $vote ) {
				//Save a transient (todo - user configurable?) for one week
				set_transient( $this->get_unique_hash( $site_id . $blog_id, $post_id ), $vote, $one_week );
			} else {
				//User has previously voted on this item - Is the vote the different?
				if ( $user_voting_record == $vote ) {
					//User can't vote again for the same item
					return array( 'error' => __( 'Cannot vote for the same item twice', 'it-l10n-thumbsup' ) );
				} 
			}
			//If user has previously voted down or up for an item and changed their mind, we want to essentially erase their vote, so let's subtract one from the total votes and what they voted for previously
			
			if ( $user_voting_record && $user_voting_record != $vote ) {
				if ( $user_voting_record == 'up' ) {
					$post_rating->up_votes -= 1;
				} else {
					$post_rating->down_votes -= 1;
				}
			}
			$post_rating->total_votes = $post_rating->up_votes + $post_rating->down_votes;
			//Make sure results aren't negative
			$post_rating->up_votes = $post_rating->up_votes < 0 ? 0 : $post_rating->up_votes;
			$post_rating->down_votes = $post_rating->down_votes < 0 ? 0 : $post_rating->down_votes;
			$post_rating->total_votes = $post_rating->total_votes < 0 ? 0 : $post_rating->total_votes;
			
			
			//Update up or down votes - $votes == up is an up vote
			if ( $vote == 'up' ) {
				$post_rating->up_votes += 1; 
				$vote = true;
			} else {
				$post_rating->down_votes += 1;
				$vote = false;
			}
			
			//Update the post
			$wpdb->update( $this->tablename, array( 'up_votes' => $post_rating->up_votes, 'down_votes' => $post_rating->down_votes ), array( 'site_id' => $site_id, 'blog_id' => $blog_id, 'content_id' => $post_id ), array( '%d', '%d' ) );
			
			//Spiffy, now let's calculate the total ratings for everything and update!
			$total_items = $wpdb->get_var( $wpdb->prepare( "select count( ID ) from {$this->tablename} where site_id = %d and blog_id = %d and post_type = '%s'", $site_id, $blog_id, $post_type ) );
			
			$results = $wpdb->get_row( $wpdb->prepare( "select SUM(up_votes + down_votes) as total_votes, SUM( ( up_votes * 5 + down_votes * 1 ) / {$total_items} ) as ratings_sum  from {$this->tablename} where site_id = %d and blog_id = %d and post_type = '%s'", $site_id, $blog_id, $post_type ) );
					
			$args = array(
				'total_items' => $total_items,
				'total_votes' => $results->total_votes,
				'average_rating' => $results->ratings_sum / $total_items,
				'average_votes' => $results->total_votes / $total_items,
			);
			
			$sql = $wpdb->prepare( "UPDATE {$this->tablename} set rating = ( ( {$args['average_votes']} * {$args['average_rating']} ) + ( ( up_votes + down_votes ) * ( up_votes * 5 + down_votes * 1 ) / ( up_votes + down_votes ) ) ) / {$args['average_votes']} + up_votes + down_votes where site_id = %d and blog_id = %d and post_type = '%s'", $site_id, $blog_id, $post_type );			
			$wpdb->query( $sql );			

			//Return Ajax response
			$return = array(
				'group_id' => $post_id,
				'success' => true,
				'thumbsup' => $vote,
				'item' => array( 'id' => $post_id ),
				'up_count' => number_format( $post_rating->up_votes ),
				'down_count' => number_format( $post_rating->down_votes )
			);
			return $return;
			
		} //end ajax_post_vote
		
		/**
		* ajax_vote()
		* 
		* Adds an up/down vote for an item
		*
		* @param		int    		$item_id		$_REQUEST[ 'item_id' ] - The item ID to be voted on
		* @param		string	$vote			$_REQUEST[ 'thumbsup' ] - 'up' or 'down'
		* @return		string(JSON)	A JSON string of the results3			
				group_id			int			The voting group ID
				thumbsup			bool		true or false (true = up, false = down)
				item					array	The item voted on
					id					int
					up_votes		int
					down_votes	int
					avg_rating	float
					total_votes	int
					rank				float
				up_count			string	Formatted  up count number to display
				down_count		string	Formatted down count number to display
		*/
		public function ajax_vote() {
			//Get variables and perform sanity checks
			$item_id = isset( $_REQUEST['item_id'] ) ? absint( $_REQUEST['item_id'] ) : false;
			$vote = isset( $_REQUEST['thumbsup'] ) ? $_REQUEST['thumbsup'] : NULL;
			if ( !$item_id ) die( json_encode( array( 'error' => __( 'Item doesn\'t exist', 'it-l10n-thumbsup' ) ) ) );
			if ( is_null( $vote ) ) die( json_encode( array( 'error' => __( 'Vote is null', 'it-l10n-thumbsup' ) ) ) );
			
			//Get item post
			$item = get_post( $item_id );
			if ( !$item ) die( json_encode( array( 'error' => __( 'Item doesn\'t exist', 'it-l10n-thumbsup' ) ) ) );
			
			//Call ajax vote_post function if the post type is global
			if ( in_array( $item->post_type, $this->get_admin_option( 'post_types' ) ) ) {
				die( json_encode( $this->ajax_post_vote( $item, $vote ) ) );
			} 
			
			//Get the parent post
			$parent_id = $item->post_parent;
			
			$ratings = $this->get_ratings( array( $item_id ), $parent_id );
			$items = &$ratings->items;
			
			//Get a transient (if it exists) to determine previous voting behavior
			$one_week = 604800;
			$user_voting_record = get_transient( $this->get_unique_hash( $parent_id, $item_id ) );
			if ( !$user_voting_record || $user_voting_record != $vote ) {
				//Save a transient (todo - user configurable?) for one week
				set_transient( $this->get_unique_hash( $parent_id, $item_id ), $vote, $one_week );
			} else {
				//User has previously voted on this item - Is the vote the different?
				if ( $user_voting_record == $vote ) {
					//User can't vote again for the same item
					die ( json_encode( array( 'error' => __( 'Cannot vote for the same item twice', 'it-l10n-thumbsup' ) ) ) );
				} 
			}
			//If user has previously voted down or up for an item and changed their mind, we want to essentially erase their vote, so let's subtract one from the total votes and what they voted for previously
			
			if ( $user_voting_record && $user_voting_record != $vote ) {
				if ( $user_voting_record == 'up' ) {
					$items[ $item_id ][ 'up_votes'] -= 1;
				} else {
					$items[ $item_id ][ 'down_votes'] -= 1;
				}
			}
			//Make sure results aren't negative
			$items[ $item_id ][ 'up_votes'] = $items[ $item_id ][ 'up_votes'] < 0 ? 0 : $items[ $item_id ][ 'up_votes'];
			$items[ $item_id ][ 'down_votes'] = $items[ $item_id ][ 'down_votes'] < 0 ? 0 : $items[ $item_id ][ 'down_votes'];
			$items[ $item_id ][ 'total_votes'] = $items[ $item_id ][ 'total_votes'] < 0 ? 0 : $items[ $item_id ][ 'total_votes'];
			
			
			//Update up or down votes - $votes == up is an up vote
			if ( $vote == 'up' ) {
				$items[ $item_id ][ 'up_votes'] += 1; 
				$vote = true;
			} else {
				$items[ $item_id ][ 'down_votes'] += 1;
				$vote = false;
			}						

			
			$item_up_votes = $items[ $item_id ][ 'up_votes'];
			$item_down_votes = $items[ $item_id ][ 'down_votes'];
			
			$this->calculate_ratings( array( 'ratings' => &$ratings, 'item_id' => $item_id, 'parent_id' => $parent_id ) );
			
			//Return Ajax response
			$return = array(
				'group_id' => absint( $parent_id ),
				'success' => true,
				'thumbsup' => $vote,
				'item' => $ratings->items[ $item_id ],
				'up_count' => number_format( $item_up_votes ),
				'down_count' => number_format( $item_down_votes )
			);
			die( json_encode( $return ) );
		} //end ajax_vote
		
		public function ajax_suggest() {
			check_ajax_referer( 'pb_thumbsup_suggest_add' );
			parse_str( $_POST['ajax_form_data'], $form_data );
			
			$post_title = sanitize_text_field( $form_data[ 'thumbsup_suggest_topic' ] );
			$group_id = absint( $form_data[ 'thumbsup_group_id' ] );
			
			//Setup post args
			$postargs = array(
				'post_title' => $post_title,
				'post_status' => 'draft',
				'post_parent' => $group_id,
				'post_type' => 'it_voting_group_item',
			);
			
			//Yay, insert post
			wp_insert_post( $postargs );
			
			//Send the admin an e-mail notification
			$notify_message  = sprintf( __('Someone has suggested a topic for ThumbsUp', 'it-l10n-thumbsup') ) .  "\r\n";
			$notify_message  .= sprintf( __('The suggested topic is: %s', 'it-l10n-thumbsup'), stripslashes( $post_title ) ) . "\r\n\r\n";
			$notify_message .= sprintf( __('View the Voting Group: %s', 'it-l10n-thumbsup'), admin_url("admin.php?post={$group_id}&action=edit&page=pluginbuddy_thumbsup-edit") ) . "\r\n";
			$email = get_option('admin_email');
			$blogname = get_option('blogname');
			$from = "From: \"$blogname\" <$email>";
			$reply_to = "Reply-To: $email";
			$message_headers = "$from\n"
				. "Content-Type: text/plain; charset=\"" . get_option('blog_charset') . "\"\n";
			$message_headers .= $reply_to . "\n";
			$subject = sprintf( __('New Topic Suggestion for ThumbsUp: "%s"','it-l10n-thumbsup'), $blogname );
			
			if ( $this->get_admin_option( 'email_notifications' ) == 'true' ) {
				@wp_mail($email, $subject, $notify_message, $message_headers);
			}

			die( json_encode( array( 'success' => __( 'Thanks for your suggestion', 'it-l10n-thumbsup' ) ) ) );
		} //end ajax_suggest
		/**
		 *	alert()
		 *
		 *	Displays a message to the user at the top of the page when in the dashboard.
		 *
		 *	$message		string		Message you want to display to the user.
		 *	$error			boolean		OPTIONAL! true indicates this alert is an error and displays as red. Default: false
		 *	$error_code		int			OPTIONAL! Error code number to use in linking in the wiki for easy reference.
		 */
		public function alert( $message, $error = false, $error_code = '' ) {
			$log_error = false;
			
			echo '<div id="message" class="';
			if ( $error === false ) {
				echo 'updated fade';
			} else {
				echo 'error';
				$log_error = true;
			}
			if ( $error_code != '' ) {
				$message .= '<p><a href="http://ithemes.com/codex/page/' . $this->get_plugin_info( 'name' ) . ':_Error_Codes#' . $error_code . '" target="_new"><i>' . $this->get_plugin_info( 'name' ) . ' Error Code ' . $error_code . ' - Click for more details.</i></a></p>';
				$log_error = true;
			}
			if ( $log_error === true ) {
				$this->log( $message . ' Error Code: ' . $error_code, 'error' );
			}
			echo '"><p><strong>' . $message . '</strong></p></div>';
		} //end alert
		
		/**
		* calculate_ratings()
		* 
		* Calculates the ratings for an item
		* Updates post meta for the item_id (pb_thumbsup_score) and ratings for the parent_id pb_thumbsup_ratings
		* The defaults for the parameter $args are:
		*     'ratings'   	object		An object of ratings (see function get_ratings for params)
		*     'item_id'		int				Item ID of the item     - Default is 'post'.
		*     'parent_id'	int				Group ID the item belong to
		*
		* @param		array    $args		$_REQUEST[ 'item_id' ] - The item ID to be voted on
		*/
		public function calculate_ratings( $args ) {
			$defaults = array('ratings' => false, 'item_id' => 0, 'parent_id' => 0 );
			extract( wp_parse_args( $args, $defaults ) );
			if ( !$ratings ) $ratings = $this->get_ratings( array( $item_id ), $parent_id );
			
			$items = &$ratings->items;
			
			$items[ $item_id ][ 'total_votes' ] = $items[ $item_id ][ 'up_votes'] + $items[ $item_id ][ 'down_votes'];
			
			if ( $items[ $item_id ] [ 'total_votes' ] <= 0 ) {
				$items[ $item_id ][ 'avg_rating' ] = 0;
			} else {
				$items[ $item_id ][ 'avg_rating' ] = ( ( $items[ $item_id ][ 'up_votes'] * $ratings->up_weight ) + ( $items[ $item_id ][ 'down_votes'] * $ratings->down_weight ) ) / $items[ $item_id ][ 'total_votes' ];
			}
			
			//Calculate the average ratings for individual items and all items
			$ratings_sum = $total_votes = 0;
			foreach ( $items as $id => &$item_data ) {
				$up_votes = $item_data[ 'up_votes' ];
				$down_votes = $item_data[ 'down_votes' ];
				$item_data[ 'total_votes' ] = $up_votes + $down_votes;
				if ( $item_data[ 'total_votes' ] <= 0 ) {
					$item_data[ 'avg_rating' ] = 0;
				} else {
					$item_data[ 'avg_rating' ] = ( ( $up_votes * $ratings->up_weight ) + ($down_votes * $ratings->down_weight ) ) / $item_data[ 'total_votes' ];
				}
				$ratings_sum += $item_data[ 'avg_rating' ];
				$total_votes = $total_votes + $item_data['total_votes'];
			}			
			$ratings->avg_rating = count( $items ) > 0 ? $ratings_sum / count( $items ) : 0;
			
			
			
			//Calculate the average number of votes for all items
			$ratings->total_votes = $total_votes;
			$ratings->avg_votes = count( $items ) > 0 ? $ratings->total_votes / count( $items ) : 0;
			
			
			//Now calculate the ratings for all the items
			foreach( $items as $id => &$item_data ) {
				$calc_numerator = ( $ratings->avg_votes * $ratings->avg_rating ) + ( $item_data[ 'total_votes' ] * $item_data[ 'avg_rating' ] );
				$calc_denominator = $ratings->avg_votes + $item_data[ 'total_votes' ];
				$item_rank = $calc_denominator > 0 ? $calc_numerator / $calc_denominator : 0;
				$item_data[ 'rank' ] = $item_rank;
				
				//Update post meta for items
				update_post_meta( $id, 'pb_thumbsup_score', $item_rank );
			} //end foreach
			//Save data
			update_post_meta( $parent_id, 'pb_thumbsup_ratings', $ratings );
		} //end calculate_ratings
		
		public function display_thumbsup_posts( $content ) {
			if ( !is_single() && !is_page() ) return $content;
			global $post;
			$post_type = get_post_type();
			if ( !in_array( $post_type, $this->get_admin_option( 'post_types' ) ) ) return $content;
			return $content . pb_thumbsup_display_posts( array() );
		} //end display_thumbsup_posts
		/**
		* get_admin_option()
		* 
		* Returns a localized admin option
		*
		* @param   string    $key    Admin Option Key to Retrieve
		* @return   mixed                The results of the admin key.  False if not present.
		*/
		public function get_admin_option( $key = '' ) {			
			$admin_options = $this->get_admin_options();
			if ( array_key_exists( $key, $admin_options ) ) {
				return $admin_options[ $key ];
			}
			return false;
		}
		
		/**
		* get_admin_options()
		* 
		* Initialize and return an array of all admin options
		*
		* @return   array					All Admin Options
		*/
		public function get_admin_options( ) {
			
			if (empty($this->admin_options)) {
				$admin_options = $this->get_plugin_defaults();
				
				$options = get_option( $this->get_plugin_info( 'slug' ) );
				if (!empty($options)) {
					foreach ($options as $key => $option) {
						if (array_key_exists($key, $admin_options)) {
							$admin_options[$key] = $option;
						}
					}
				}
				
				//Save the options
				$this->admin_options = $admin_options;
				$this->save_admin_options();								
			}
			return $this->admin_options;
		} //end get_admin_options
		
		/**
		* get_all_admin_options()
		* 
		* Returns an array of all admin options
		*
		* @return   array					All Admin Options
		*/
		public function get_all_admin_options() {
			return $this->admin_options;
		}
		
		/**
		*	get_item_vote()
		*
		*	Returns a transient for the item based on the user - Uses get_unique_hash
		*
		*	$item_id		int								The Item ID 
		*	@return		string or bool			"up" or "down" (false on failure)
		*/
		public function get_item_vote( $parent_id, $item_id ) {
			$transient = get_transient( $this->get_unique_hash( intval( $parent_id ), intval( $item_id ) ) );
			return $transient;
		} //end get_item_vote
		
		/**
		* get_plugin_defaults()
		* 
		* Returns an array of default plugin options (to be stored in the options table)
		*
		* @return		array               Default plugin keys
		*/
		public function get_plugin_defaults() {
			if ( isset( $this->defaults ) ) {
				return $this->defaults;
			} else {
				$this->defaults = array(
					'key' => '',
					'guid' => '',
					'thumbs_up_style' => 'light',
					'disable_styles' => "false",
					'post_types' => array(),
					'post_voting_text' => 'Rate this &#x279C;',
					'post_voting_display' => 'both',
					'table_version' => '1.0',
					'table_exists' => 'false',
					'suggest_widgets' => array(),
					'email_notifications' => 'true'
				);
				return $this->defaults;
			}
		} //end get_plugin_defaults
		
		/**
		* get_plugin_dir()
		* 
		* Returns an absolute path to a plugin item
		*
		* @param		string    $path	Relative path to make absolute (e.g., /css/image.png)
		* @return		string               An absolute path (e.g., /htdocs/ithemes/wp-content/.../css/image.png)
		*/
		public function get_plugin_dir( $path = '' ) {
			$dir = $this->plugin_dir;
			if ( !empty( $path ) && is_string( $path) )
				$dir .= '/' . ltrim( $path, '/' );
			return $dir;		
		} //end get_plugin_dir
			
		/**
		* get_plugin_info()
		* 
		* Returns a localized plugin key
		*
		* @param   string    $key    Plugin Key to Retrieve
		* @return   mixed                The results of the plugin key.  False if not present.
		*/
		public function get_plugin_info( $key = '' ) {	
			if ( array_key_exists( $key, $this->plugin_info ) ) {
				return $this->plugin_info[ $key ];
			}
			return false;
		} //end get_plugin_info
		
		
		/**
		* get_plugin_url()
		* 
		* Returns an absolute url to a plugin item
		*
		* @param		string    $path	Relative path to plugin (e.g., /css/image.png)
		* @return		string               An absolute url (e.g., http://www.domain.com/plugin_url/.../css/image.png)
		*/
		public function get_plugin_url( $path = '' ) {
			$dir = $this->plugin_url;
			if ( !empty( $path ) && is_string( $path) )
				$dir .= '/' . ltrim( $path, '/' );
			return $dir;	
		} //get_plugin_url
		
		/**
		* get_unique_hash()
		* 
		* Saves a hash of server data that is more-or-less unique to the user
		* @param		int			$group_id	The Group ID to retrieve the hash for
		* @param		int    		$item_id		The item ID to retrieve the hash for
		* @return		string	The unique user hash
		*/
		private function get_unique_hash( $group_id, $item_id ) {
			if ( !$this->user_hash ) {
				$ip = isset( $_SERVER['REMOTE_ADDR'] ) ? $_SERVER['REMOTE_ADDR'] : '';
				$http_charset = isset( $_SERVER['HTTP_ACCEPT_CHARSET'] ) ? $_SERVER['HTTP_ACCEPT_CHARSET'] : '';
				$http_encoding = isset( $_SERVER['HTTP_ACCEPT_ENCODING'] ) ? $_SERVER['HTTP_ACCEPT_ENCODING'] : '';
				$http_lang = isset( $_SERVER['HTTP_ACCEPT_LANGUAGE'] ) ? $_SERVER['HTTP_ACCEPT_LANGUAGE'] : '';
				$http_ua = isset( $_SERVER['HTTP_USER_AGENT'] ) ? $_SERVER['HTTP_USER_AGENT'] : '';
				$this->user_hash = md5( $ip . $http_charset . $http_encoding . $http_lang . $http_ua );
			}
			
			$hash = "pb_{$group_id}{$item_id}{$this->user_hash}";
			return $hash;
		} //end get_unique_hash
		
		//Return the post stats for a post
		public function get_post_stats( $post_id = 0) {
			global $current_user, $current_blog, $post, $wpdb, $is_winIE;
			$site_id = $blog_id = 1;
			if ( is_multisite() ) {
				$site_id = $current_blog->site_id;
				$blog_id = $current_blog->blog_id;
			}
			if ( is_object( $post ) && $post_id <= 0 ) {
				$post_id = $post->ID;
			}
			if ( $post_id <= 0 ) return false;
			$sql = "select * from {$this->tablename} where content_id = %d and site_id = %d and blog_id = %d";
			$results = $wpdb->get_row( $wpdb->prepare( $sql, $post_id, $site_id, $blog_id ) );
			if ( $results ) {
				return $results;
			}
			return false;
		} //end get_post_stats
		public function get_tablename() {
			return $this->tablename;
		} //end get_tablename
		/**
		* init()
		* 
		* Initializes plugin localization, post types, updaters, plugin info, and adds actions/filters
		*
		*/
		function init() {		
			
						

			//* Localization Code */
			load_plugin_textdomain( 'it-l10n-thumbsup', false, dirname( plugin_basename( __FILE__ ) ) . '/lang/' );
			
			//Upgrader
			if ( is_admin() ) { // Runs when in the admin dashboard.
				require_once( $this->get_plugin_dir( '/classes/admin.php' ) );
				add_action( 'init', array( &$this, 'upgrader_register' ), 50 );
				add_action( 'init', array( &$this, 'upgrader_select' ), 100 );
				add_action( 'init', array( &$this, 'upgrader_instantiate' ), 101 );
				//register_activation_hook( __FILE__, array( &$this, 'activate' ) ); // Run some code when plugin is activated in dashboard.
			}
			
			//Add plugin info
			require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
			$this->plugin_info = wp_parse_args( array_change_key_case( get_plugin_data( __FILE__, false, false ), CASE_LOWER ), $this->plugin_info );
			
			////Create our own version of the_content so that others can't accidentally loop into our output - Taken from default-filters.php, shortcodes.php, and media.php
			if ( !has_filter( 'pb_the_content', 'wptexturize' ) ) {
				add_filter( 'pb_the_content', 'wptexturize'        );
				add_filter( 'pb_the_content', 'convert_smilies'    );
				add_filter( 'pb_the_content', 'convert_chars'      );
				add_filter( 'pb_the_content', 'wpautop'            );
				add_filter( 'pb_the_content', 'shortcode_unautop'  );
				add_filter( 'pb_the_content', 'prepend_attachment' );
				$vidembed = new WP_Embed();
				add_filter( 'pb_the_content', array( &$vidembed, 'run_shortcode'), 8 );
				add_filter( 'pb_the_content', array( &$vidembed, 'autoembed'), 8 );
				add_filter( 'pb_the_content', 'do_shortcode', 11);
			} //end has_filter
			
			//Initialize post types
			$args = array(
				'labels' => array(),
				'public' => true,
				'publicly_queryable' => true,
				'show_ui' => false, 
				'rewrite' => true,
				'query_var' => true,
				'capability_type' => 'post',
				'exclude_from_search' => true,
				'hierarchical' => false,
				'show_in_nav_menus' => false,
				'menu_position' => 1000,
				'supports' => array(
					'title',  'thumbnail', 'editor', 'excerpt', 'custom-fields', 'page-attributes'
				)
			);
			register_post_type('it_voting_group',$args);
			
			$args = array(
				'labels' => array(),
				'public' => true,
				'publicly_queryable' => true,
				'show_ui' => false, //false - to hide from the admin area menu
				'rewrite' => false,
				'query_var' => true,
				'capability_type' => 'post',
				'exclude_from_search' => true,
				'hierarchical' => true, //allow parent pages
				'show_in_nav_menus' => false,
				'menu_position' => 1000,
				'supports' => array(
					'title',  'thumbnail', 'editor', 'excerpt', 'custom-fields', 'page-attributes'
				)
			);
			register_post_type('it_voting_group_item',$args);
			
			//Ajax
			add_action( 'wp_ajax_nopriv_pb_thumbs_up', array( &$this, 'ajax_vote' ) );
			add_action( 'wp_ajax_pb_thumbs_up', array( &$this, 'ajax_vote' ) );
			add_action( 'wp_ajax_nopriv_pb_thumbsup_suggest', array( &$this, 'ajax_suggest' ) );
			add_action( 'wp_ajax_pb_thumbsup_suggest', array( &$this, 'ajax_suggest' ) );
			
			//Media bar
			add_action('media_buttons_context', array( &$this, 'add_media_button') );
			add_action( 'admin_footer-post.php', array( &$this, 'add_thumbsup_shortcode' ) );
			add_action( 'admin_footer-post-new.php', array( &$this, 'add_thumbsup_shortcode' ) );
			
			//Content for post types
			add_filter( 'the_content', array( &$this, 'display_thumbsup_posts' ) );
			add_action( 'wp_print_scripts', array( &$this, 'add_post_scripts' ) );
			add_action( 'wp_print_styles', array( &$this, 'add_post_styles' ) );
			
			//For widget control
			add_filter('sidebars_widgets', array( &$this, 'remove_widgets' ) );
			
			//shortcode
	   		add_shortcode( 'thumbsup', array( &$this, 'output_thumbsup_shortcode' ) ); 
	   		
	   		//Create the posts table if the table doesn't exists or needs updating
	   		
	   		if ( $this->get_admin_option( 'table_exists' ) == 'false' || version_compare( $this->get_admin_option( 'table_version' ), $this->table_version, '<' ) ) {
	   			global $wpdb;
	   			if ( is_multisite() ) switch_to_blog( 1 );
	   			
	   			$table = $this->tablename;	
				$sql = "CREATE TABLE " . $this->tablename . " (
						id INT(20) NOT NULL AUTO_INCREMENT,
						site_id INT (20) NOT NULL,
						blog_id INT (20) NOT NULL,
						author_id INT (20) NOT NULL,
						content_id INT(20) NOT NULL,
						post_type VARCHAR(20) NOT NULL,
						up_votes INT (20) NOT NULL,
						down_votes INT (20) NOT NULL,
						rating FLOAT (20) NOT NULL,
						UNIQUE KEY id (id)
					);";
				require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
				dbDelta($sql);
				$this->save_admin_option( 'table_version', $this->table_version );
	   			if ( is_multisite() ) restore_current_blog();
	   			
	   		} //end create table
			
		}//end function init
		
		
		
		/**
		* get_ratings()
		* 
		* Retrieves ratings for a voting group( $parent_id ) and adds defaults
		*
		* @param		array    $item_ids		Group of item IDs to add defaults for ( optional )
		* @param		int			$parent_id	Group ID to retrieve the ratings for
		* @return		stdClass	An object with the ratings
				avg_votes		float
				avg_rating		float
				up_weight		int
				down_weight	int
				total_votes		int
				items				array
					id					int
					up_votes		int
					down_votes	int
					avg_rating	float
					total_votes	int
					rank				float
				
		*/
		public function get_ratings( $item_ids = array(), $parent_id ) {
			//Now get the ratings - Create structure if it doesn't exist
			$ratings = get_post_meta( $parent_id, 'pb_thumbsup_ratings', true );
			if ( !$ratings ) {
				//No ratings, so let's create a base structure
				$ratings = new stdClass;
				$ratings->avg_votes = 0;
				$ratings->avg_rating = 0;
				$ratings->up_weight = 5;
				$ratings->down_weight = 1;
				$ratings->total_votes = 0;
				$ratings->items = array();
			}
			
			//Now get the item if applicable
			$items = &$ratings->items;
			foreach ( $item_ids as $item_id ) {
				if ( !isset( $items[ $item_id ] ) ) {
					$items[ $item_id ] = array(
						'id' => $item_id,
						'up_votes' => 0,
						'down_votes' => 0,
						'avg_rating' => 0,
						'total_votes' => 0,
						'rank' => 0
					);
				}
			}
				
			return $ratings;
		} //end get_ratings
		
		/**
		 *	log()
		 *
		 *	Logs to a text file depending on settings.
		 *	0 = none, 1 = errors only, 2 = errors + warnings, 3 = debugging (all kinds of actions)
		 *
		 *	$text	string			Text to log.
		 *	$log_type	string		Valid options: error, warning, all (default so may be omitted).
		 *
		 */
		public function log( $text, $log_type = 'all' ) {
			$write = false;
			
			if ( $this->get_plugin_option( 'log_level' ) == 0 ) { // No logging.
				return;
			} elseif ( $this->get_plugin_option( 'log_level' ) == 1 ) { // Errors only.
				if ( $log_type == 'error' ) {
					$write = true;
				}
			} elseif ( $this->get_plugin_option( 'log_level' ) == 2 ) { // Errors and warnings only.
				if ( ( $log_type == 'error' ) || ( $log_type == 'warning' ) ) {
					$write = true;
				}
			} elseif ( $this->get_plugin_option( 'log_level' ) == 3 ) { // Log all; Errors, warnings, actions, notes, etc.
				$write = true;
			}
			
			if ( $write === true ) {
				$fh = fopen( WP_CONTENT_DIR . '/uploads/' . $this->get_plugin_info( 'slug' ) . '.txt', 'a');
				fwrite( $fh, '[' . date( $this->_timestamp . ' ' . get_option( 'gmt_offset' ), time() + (get_option( 'gmt_offset' )*3600) ) . '-' . $log_type . '] ' . $text . "\n" );
				fclose( $fh );
			}
		} //end log
		
		public function remove_widgets( $sidebars ) {
			$suggest_widgets = $this->get_admin_option( 'suggest_widgets' );
			foreach ( $suggest_widgets as $widget_id => $widget_data ) {
				if ( isset( $widget_data[ 'logged_in' ] ) ) {
					if ( $widget_data[ 'logged_in' ] == 'true' && !is_user_logged_in() ) {
						//User isn't logged in and the widget requires the person to be logged in - Let's find the widget and unset it
						foreach ( $sidebars as $sidebar => $sidebar_widgets ) {
							foreach ( $sidebar_widgets as $key => $widget_name ) {
								if ( $widget_name == $widget_id ) {
									unset( $sidebars[ $sidebar ][ $key ] );
								}
							} //end foreach $sidebar_widgets
						} //end foreach $sidebars
					} //end if user logged in
				}
			} //end foreach $suggest_widgets
			return $sidebars;
		} //end remove_widgets
		
		/**
		* output_thumbsup_shortcode()
		* 
		* Outputs a shortcode - uses pb_thumbsup_display
		*
		* @param		array    $atts	Shortcode attributes (see pb_thumbsup_display for params)
		* @return		string    Shortcode output (or empty string on failure)
		*/
		public function output_thumbsup_shortcode( $atts ) {
			$display = pb_thumbsup_display( $atts );
			if ( !is_wp_error( $display ) ) {
				return $display;
			}
			return '';
		} //end output_thumbsup_shortcode
		
		/**
		* save_admin_option()
		* 
		* Saves an individual option to the options array
		* @param		string    	$key		Option key to save
		* @param		mixed		$value	Value to save in the option	
		*/
		public function save_admin_option( $key = '', $value = '' ) {
			$this->admin_options[ $key ] = $value;
			$this->save_admin_options();
			return $value;
		} //end save_admin_option
		
		/**
		* save_admin_options()
		* 
		* Saves a group of admin options to the options table
		* @param		array    	$admin_options		Optional array of options to save (are merged with existing options)
		*/
		public function save_admin_options( $admin_options = false ){
			if (!empty($this->admin_options)) {
				if ( is_array( $admin_options ) ) {
					$this->admin_options = wp_parse_args( $admin_options, $this->admin_options );
				}
				update_option( $this->get_plugin_info( 'slug' ), $this->admin_options);
			}
		} //end save_admin_options
		
		//Save the post stats for a post
		public function save_post_stats( $post_id = 0, $meta ) {
			global $current_user, $current_blog, $post, $wpdb, $is_winIE;
			$site_id = $blog_id = 1;
			if ( is_multisite() ) {
				$site_id = $current_blog->site_id;
				$blog_id = $current_blog->blog_id;
			}
			if ( is_object( $post ) && $post_id <= 0 ) {
				$post_id = $post->ID;
			}
			if ( $post_id <= 0 ) return false;
			$sql = "select * from {$this->tablename} where content_id = %d and site_id = %d and blog_id = %d";
			$results = $wpdb->get_results( $wpdb->prepare( $sql, $post_id, $site_id, $blog_id ) );
			if ( $results ) {
				return $results->meta;
			}
			return false;
		} //end get_post_stats
		
		
		
		
		/**
		 *	tip()
		 *
		 *	Displays a message to the user when they hover over the question mark. Gracefully falls back to normal tooltip.
		 *	HTML is supposed within tooltips.
		 *
		 *	$message		string		Actual message to show to user.
		 *	$title			string		Title of message to show to user. This is displayed at top of tip in bigger letters. Default is blank. (optional)
		 *	$echo_tip		boolean		Whether to echo the tip (default; true), or return the tip (false). (optional)
		 */
		public function tip( $message, $title = '', $echo_tip = true ) {
			$tip = ' <a class="pluginbuddy_tip" title="' . $title . ' - ' . $message . '"><img src="' . $this->get_plugin_url( '/images/pluginbuddy_tip.png' ) . '" alt="(?)" /></a>';
			if ( $echo_tip === true ) {
				echo $tip;
			} else {
				return $tip;
			}
		} //end tip
		
		/**
		 *	video()
		 *
		 *	Displays a message to the user when they hover over the question mark. Gracefully falls back to normal tooltip.
		 *	HTML is supposed within tooltips.
		 *
		 *	$video_key		string		YouTube video key from the URL ?v=VIDEO_KEY_HERE
		 *	$title			string		Title of message to show to user. This is displayed at top of tip in bigger letters. Default is blank. (optional)
		 *	$echo_tip		boolean		Whether to echo the tip (default; true), or return the tip (false). (optional)
		 */
		public function video( $video_key, $title = '', $echo_tip = true ) {
			global $wp_scripts;
			if ( !in_array( 'thickbox', $wp_scripts->done ) ) {
				wp_enqueue_script( 'thickbox' );
				wp_print_scripts( 'thickbox' );
				wp_print_styles( 'thickbox' );
			}
			
			$tip = '<a href="http://www.youtube.com/embed/' . urlencode( $video_key ) . '?autoplay=1&TB_iframe=1&width=640&height=400" class="thickbox pluginbuddy_tip" title="Video Tutorial - ' . $title . '"><img src="' . $this->get_plugin_url( '/images/pluginbuddy_play.png' ) . '" alt="(video)" /></a>';
			if ( $echo_tip === true ) {
				echo $tip;
			} else {
				return $tip;
			}
		} //end video
		
		//Register the updater version
		public function upgrader_register() {
			$GLOBALS['pb_classes_upgrade_registration_list'][$this->_var] = $this->_updater;
		} //end register_upgrader
		//Select the greatest version
		public function upgrader_select() {
			if ( !isset( $GLOBALS[ 'pb_classes_upgrade_registration_list' ] ) ) {
				//Fallback - Just include this class
				require_once( $this->_pluginPath . '/lib/updater/updater.php' );
				return;
			}
			//Go through each global and find the highest updater version and the plugin slug
			$updater_version = 0;
			$plugin_var = '';
			foreach ( $GLOBALS[ 'pb_classes_upgrade_registration_list' ] as $var => $version) {
				if ( version_compare( $version, $updater_version, '>=' ) ) {
					$updater_version = $version;
					$plugin_var = $var;
				}
			}
			//If the slugs match, load this version
			if ( $this->_var == $plugin_var ) {
				require_once( $this->_pluginPath . '/lib/updater/updater.php' );
			}
		} //end upgrader_select
		public function upgrader_instantiate() {
			
			$pb_product = strtolower( $this->_var );
			$pb_product = str_replace( 'ithemes-', '', $pb_product );
			$pb_product = str_replace( 'pluginbuddy-', '', $pb_product );
			$pb_product = str_replace( 'pluginbuddy_', '', $pb_product );
			$pb_product = str_replace( 'pb_thumbsup', '', $pb_product );
			
			$args = array(
				'parent' => $this, 
				'remote_url' => 'http://updater2.ithemes.com/index.php',
				'version' => $this->_version,
				'plugin_slug' => $this->_var,
				'plugin_path' => plugin_basename( __FILE__ ),
				'plugin_url' => $this->_pluginURL,
				'product' => $pb_product,
				'time' => 43200,
				'return_format' => 'json',
				'method' => 'POST',
				'upgrade_action' => 'check' );
			$this->_pluginbuddy_upgrader = new iThemesPluginUpgrade( $args );

		} //end upgrader_instantiate
		
    } //end class
}
//instantiate the class
global $pb_thumbsup;
if (class_exists('pluginbuddy_thumbsup')) {
	if (get_bloginfo('version') >= "3.0") {
		add_action( 'plugins_loaded', 'pb_thumbsup_instantiate' );
	}
}
function pb_thumbsup_instantiate() {
	global $pb_thumbsup;
	$pb_thumbsup = new pluginbuddy_thumbsup();
	include_once( $pb_thumbsup->get_plugin_dir( 'classes/widget.thumbsup.php' ) );
}
/**
* pb_thumbsup_display()
* Insert a voting group
*
*
* The defaults for the parameter $args are:
*		display - none, both, up, down - Whether to show up/down voting
*		is_widget - If the code is being executed in a widget area (shrinks down the icon size) (default false)
*		show_stats - Whether to show the stats or not (default true)
*		group_id   - The ID of the voting group you would like to display.
*		thumb_width     - Default is 200. The Width dimension for displaying image thumbnails.
*		thumb_height   - Default is 200. The Height dimension for displaying image thumbnails.
*		video_max_height - Default is 300. The maximum height for an embedded video
*		video_max_width - Default is 300. The maximum width for an embedded video
*		show_item_title - Default is true. Whether to show the title for each item
*		show_item_desc - Default is true.  Whether to show the item description for each item
*		show_group_title - Default is true. Whether to display the title for the voting group
*		show_group_desc - Default is false. Whether to display the description for the voting group
*		show_image_caption - Default is false. Whether to display the caption for an image
*		show_image_desc - Default is false. Whether to display the description for an image
*		orderby - Default is 'menu_order' - Which parameter to order by (rating, date, menu_order, rand)
*		order - Default is 'DESC'. The order to retrieve the items.
*
* @uses $wpdb
*
* @param array $args Elements that make up a voting group to insert.
* @return int|WP_Error WP_Error on failure.  HTML string on success.
*/
function pb_thumbsup_display( $args ) {
	global $post, $pb_thumbsup;
	$defaults = array(
		'display' => 'both',
		'is_widget' => false,
		'show_stats' => true,
		'group_id' => 0,
		'thumb_width' => 200,
		'thumb_height' => 200,
		'video_max_height' => 300,
		'video_max_width' => 300,
		'show_item_title' => true,
		'show_item_desc' => true,
		'show_group_title' => true,
		'show_group_desc' => false,
		'show_image_caption' => false,
		'show_image_desc' => false,
		'orderby' => 'menu_order',
		'order' => 'DESC'
	);
	$defaults = wp_parse_args( $args, $defaults );
	extract( $defaults, EXTR_SKIP );
	
	$voting_display_class = 'pb_' . esc_attr( $display ); 
	
	//Get the voting group
	$voting_group = get_post( $group_id );
	if ( !$voting_group ) return new WP_Error( 'pb_not_found', __( 'Voting group not found.', 'it-l10n-thumbsup' ) );
	
	//Get media type and expiration
	$media_type = get_post_meta( $group_id, 'media_type', true );
	if ( !$media_type ) return new WP_Error( 'pb_no_media', __( 'Media type not found.', 'it-l10n-thumbsup' ) );
	$has_expiration = get_post_meta( $group_id, 'expires', true );
	$expiration_timestamp = get_post_meta( $group_id, 'expiration_timestamp', true );
	
	//Get the posts, err, items
	$item_args = array( 
		'numberposts' => -1,
		'posts_per_page' => -1, 
		'post_parent' => $group_id, 
		'post_type' => 'it_voting_group_item',
		'order' => $order
	);
	switch ( $orderby ) {
		case 'menu_order':
		case 'rand':
		case 'date':
			$item_args['orderby'] = $orderby;
			break;
		case 'rating':
			$item_args['orderby'] = 'meta_value_num';
			$item_args['meta_key'] = 'pb_thumbsup_score';
			break;
	} //end switch $order

	$group_items = get_posts( $item_args );
	
	if ( !$group_items ) return new WP_Error( 'pb_no_items', __( 'There is nothing to vote on.', 'it-l10n-thumbsup' ) );
	
	//Footer Ajax loader
	add_action( 'wp_footer', array( &$pb_thumbsup, 'add_footer_html' ) );
			
	//Begin outputting HTML
	$widget = $is_widget ? 'widget_' : '';
	$html = "<div class='pb_thumbsup_{$widget}container'>";
	//Loading gif
	
	if ( $show_group_title ) {
		$html .= sprintf( "<h2>%s</h2>", esc_html( stripslashes( apply_filters( 'get_the_title', $voting_group->post_title ) ) ) );
	}
	if ( $show_group_desc ) {
		$html .= apply_filters( 'pb_the_content', $voting_group->post_content );
	}
	$html .= "<div class='pb_thumbsup_items'>";
	
	//Get the item stats
	$item_stats = get_post_meta( $group_id, 'pb_thumbsup_ratings', true );
	if ( $item_stats ) {
		if ( count( $item_stats->items ) > 0 ) $item_stats = $item_stats->items;
	}
	
	//Go through and display
	foreach ( $group_items as $item ) {
		$html .= sprintf( "<div class='pb_thumbsup_item' id='pb_thumbsup_item_%d'>", absint( $item->ID ) ). PHP_EOL;
		//Displaying the item title
		if ( $media_type == 'topics' || ($media_type != 'topics' && $show_item_title ) ) {
			$html .= sprintf( "<h3>%s</h3>", esc_html( stripslashes( apply_filters( 'get_the_title', $item->post_title ) ) ) ). PHP_EOL;
		} 
		//Displaying the item description (for topics)
		if ( $media_type == 'topics'  && $show_item_desc ) {
			$html .= sprintf( "<div class='pb_thumbsup_item_desc'>%s</div>", stripslashes( apply_filters( 'pb_the_content', $item->post_content )  ) ). PHP_EOL;
		} 
		if ( $media_type == 'images' || $media_type == 'videos' ) {
			$html .= "<div class='pb_thumbsup_media'>". PHP_EOL;
				if ( $media_type == 'videos' ) {
					$html .= wp_oembed_get( trim( $item->post_content ), array( 'width' => $video_max_width, 'height' => $video_max_height ) );
				} /* end video */ elseif ( $media_type == 'images' ) {
					$attachment_id = get_post_meta( $item->ID, '_thumbnail_id', true );
					$attachment_image = wp_get_attachment_link( $attachment_id , array( $thumb_width, $thumb_height ) );
					$attachment = get_post( $attachment_id );
					if ( $show_image_caption ) {
						$attachment_data = wp_get_attachment_image_src( $attachment_id, array( $thumb_width, $thumb_height ) );
						if ( $attachment_data && $attachment ) {
							$attachment_width = $attachment_data[ 1 ];
							$caption = sprintf( "[caption id='attachment_%d' align='alignnone' width='%d' caption='%s']{$attachment_image}[/caption]" , absint( $attachment_id ), absint( $attachment_width ), esc_attr( apply_filters( 'get_the_title', $attachment->post_title ) ) );
							$html .= do_shortcode( $caption );
						} //end if $attachment_data
					} else {
						$html .= $attachment_image;
					} //end if $caption
					if ( $show_image_desc && $attachment ) {
						$html .= "<div class='pb_thumbsup_media_desc'>". PHP_EOL;
						$content = apply_filters('pb_the_content', $attachment->post_content );
						$content = str_replace(']]>', ']]&gt;', $content);
						$html .= apply_filters( 'pb_the_content', $content );


						$html .= "</div><!--/.pb_thumbsup_media_desc-->". PHP_EOL;
					} //end if $description		
				} /* end images */ 
			$html .= "</div><!--/.pb_thumbsup_media-->". PHP_EOL;
		}
		
		//Show thumbsup icons
		$icon_class_suffix = $pb_thumbsup->get_admin_option( 'thumbs_up_style' );
		if ( $icon_class_suffix != 'light' && $icon_class_suffix != 'dark' ) $icon_class_suffix = 'light';
		
		//Check to see if user has already voted
		$item_vote = $pb_thumbsup->get_item_vote( $group_id, $item->ID );
		$thumbs_up_class = $thumbs_down_class = '';
		if ( false != ( $item_vote = $pb_thumbsup->get_item_vote( $group_id, $item->ID ) ) ) {
			if ( $item_vote == 'up' )
				$thumbs_up_class = 'pb_thumbsup_selected';
			else
				$thumbs_down_class = 'pb_thumbsdown_selected';
		}
		
		$thumbs_up_url = esc_url( add_query_arg( array( 'group_id' => absint( $group_id ), 'item_id' => absint( $item->ID ) ), admin_url( 'admin-ajax.php' ) ) );
		$html .= "<div class='pb_thumbsup_vote {$voting_display_class}'>". PHP_EOL;
		
		$has_expired = $has_expiration && $expiration_timestamp <= time() ? true : false;
		$html .= sprintf( "<div class='pb_thumbsup_vote_icons pb_thumbsup_{$icon_class_suffix} %s' id='pb_thumbsup_vote_icons_%d'>", $has_expired ? 'pb_thumbsup_expired' : '', absint( $item->ID ) ) . PHP_EOL;
		
		//Don't show thumbs up icons if group has expired
		if ( ( !$has_expiration || !$has_expired ) ) {
			if ( $display == 'both' || $display == 'up' ) {
				$html .= sprintf( "<a class='pb_thumbsup_icon%s' href='%s' onclick='jQuery.pb_thumbsup.vote( this, \"up\" ); return false;' id='pb_thumbsup_icon_%d'>%s</a>", ' ' . $thumbs_up_class, $thumbs_up_url, $item->ID, __( 'Thumbs Down', 'it-l10n-thumbsup' ) ). PHP_EOL;
			}
			if ( $display == 'both' || $display == 'down' ) {
				$html .= sprintf( "<a class='pb_thumbsdown_icon%s' href='%s' onclick='jQuery.pb_thumbsup.vote( this, \"down\" );  return false;' id='pb_thumbsdown_icon_%d'>%s</a>", ' ' . $thumbs_down_class, $thumbs_up_url, $item->ID, __( 'Thumbs Down', 'it-l10n-thumbsup' ) ). PHP_EOL;
			}
		}
		
		//Item stats
		$item_data = isset( $item_stats[ $item->ID ] ) ? $item_stats[ $item->ID ] : false;
		$up_votes = $down_votes = 0;
		if ( $item_data ) {
			//todo - have an option to display these up front in the next version
			$up_votes = intval( $item_data['up_votes'] );
			$down_votes = intval( $item_data[ 'down_votes' ] );
			//$html .= $item_data['rank'];
		}
		//Show stats if group has expired
		if ( $has_expiration && $expiration_timestamp <= time() ) $show_stats = true;
		$html .= "</div><!--/.pb_thumbsup_vote_icons-->". PHP_EOL;
		$html .= sprintf( "<div class='pb_thumbsup_vote_stats%s' id='pb_thumbsup_vote_stats_%d'>", $show_stats ? ' pb_thumbsup_display_stats' : '', esc_attr( $item->ID ) ). PHP_EOL;
		if ( $display == 'both' || $display == 'up' ) {
			$html .= sprintf( "&nbsp;<span class='plus'>%d</span>&nbsp;Up", $up_votes ). PHP_EOL;
		}
		if ( $display == 'both' || $display == 'down' ) {
			$html .= sprintf( "&nbsp;<span class='minus'>%d</span>&nbsp;Down", $down_votes ). PHP_EOL;
		}
		$html .= "</div><!--/.pb_thumbsup_vote_stats-->". PHP_EOL;
		$html .= "</div><!--/.pb_thumbsup_vote-->". PHP_EOL;
		
		$html .= "</div><!--/.pb_thumbsup_item-->". PHP_EOL;
	} //end foreach $group_items
	
	
	
	$html .= "</div><!--/.pb_thumbsup_items-->";
	$html .= "</div><!--/#pb_thumbsup_container-->";
	
	//Before returning, let's queue our style - The solution here isn't ideal, but it's an issue of performance vs. validation 
	if ( $pb_thumbsup->get_admin_option( 'disable_styles' ) != 'true' ) {
		wp_enqueue_style( 'pb_thumbsup_css' );
		if ( !wp_style_is( 'pb_thumbsup_css', 'done' ) ) wp_print_styles( array( 'pb_thumbsup_css' ) );
	}
	
	//Let's also enqueue our script and hope the themer had good enough sense to include the wp_footer do_action
	wp_enqueue_script( 'pb_thumbsup_js' );
	if ( !wp_script_is( 'pb_thumbsup_js', 'done' ) ) wp_print_scripts( array( 'pb_thumbsup_js' ) );
	
	//return output
	return $html;
} //end pb_thumbsup_display

/**
* pb_thumbsup_display_posts()
* Insert a voting interface to posts
*
*
* @uses $wpdb
*
* @param array $args Elements that make up a voting group to insert.
* @return int|WP_Error WP_Error on failure.  HTML string on success.
*/
function pb_thumbsup_display_posts( $args ) {
	global $post, $pb_thumbsup, $current_blog, $wpdb;
	$site_id = $blog_id = 1;
	if ( is_multisite() ) {
		$site_id = $current_blog->site_id;
		$blog_id = $current_blog->blog_id;
	}
		
	//Get the stats from the post
	$post_stats = $pb_thumbsup->get_post_stats();
	$up_votes = $down_votes = 0;
	if ( $post_stats ) {
		$up_votes = $post_stats->up_votes;
		$down_votes = $post_stats->down_votes;
	}
	$display = $pb_thumbsup->get_admin_option( 'post_voting_display' );
	$voting_display_class = 'pb_' . esc_attr( $display );
	$html = "<div class='pb_thumbsup_post_container'>";
	
	$html .= sprintf( "<div class='pb_thumbsup_item' id='pb_thumbsup_item_%d'>", absint( $post->ID ) ). PHP_EOL;
		
	//Show thumbsup icons
	$icon_class_suffix = $pb_thumbsup->get_admin_option( 'thumbs_up_style' );
	if ( $icon_class_suffix != 'light' && $icon_class_suffix != 'dark' ) $icon_class_suffix = 'light';
	
	//Check to see if user has already voted
	//$item_vote = $pb_thumbsup->get_item_vote( $group_id, $item->ID );
	$thumbs_up_class = $thumbs_down_class = '';
	if ( false != ( $item_vote = $pb_thumbsup->get_item_vote( $site_id . $blog_id, $post->ID ) ) ) {
		if ( $item_vote == 'up' )
			$thumbs_up_class = 'pb_thumbsup_selected';
		else
			$thumbs_down_class = 'pb_thumbsdown_selected';
	}
	
	$thumbs_up_url = esc_url( add_query_arg( array( 'item_id' => absint( $post->ID ) ), admin_url( 'admin-ajax.php' ) ) );
	$html .= "<div class='pb_thumbsup_vote {$voting_display_class}'>". PHP_EOL;
	$html .= sprintf( '<p>%s</p>',  $pb_thumbsup->get_admin_option( 'post_voting_text' ) );
	$html .= sprintf( "<div class='pb_thumbsup_vote_icons pb_thumbsup_{$icon_class_suffix} %s' id='pb_thumbsup_vote_icons_%d'>", '', absint( $post->ID ) ) . PHP_EOL;
	
	if ( $display == 'both' || $display == 'up' ) {
		$html .= sprintf( "<a class='pb_thumbsup_icon%s' href='%s' onclick='jQuery.pb_thumbsup.vote( this, \"up\" ); return false;' id='pb_thumbsup_icon_%d'>%s</a>", ' ' . $thumbs_up_class, $thumbs_up_url, $post->ID, __( 'Thumbs Down', 'it-l10n-thumbsup' ) ). PHP_EOL;
	}
	if ( $display == 'both' || $display == 'down' ) {
		$html .= sprintf( "<a class='pb_thumbsdown_icon%s' href='%s' onclick='jQuery.pb_thumbsup.vote( this, \"down\" );  return false;' id='pb_thumbsdown_icon_%d'>%s</a>", ' ' . $thumbs_down_class, $thumbs_up_url, $post->ID, __( 'Thumbs Down', 'it-l10n-thumbsup' ) ). PHP_EOL;
	}
	
	//Item stats
	$html .= "</div><!--/.pb_thumbsup_vote_icons-->". PHP_EOL;
	$html .= "</div><!--/.pb_thumbsup_vote-->". PHP_EOL;
	$html .= sprintf( "<div class='pb_thumbsup_vote_stats' id='pb_thumbsup_vote_stats_%d'>", esc_attr( $post->ID ) ). PHP_EOL;
	if ( $up_votes > 0 && ( $display == 'both' || $display == 'up' ) ) {
		$html .= "<span class='plus'>" . number_format( $up_votes ) . "</span> " . sprintf( "<span id='pb_thumbsup_like_%d'>", absint( $post->ID ) ) .  _n( 'person', 'people', $up_votes, 'it-l10n-thumbsup' ) . ' ' .  _n( 'likes', 'like', $up_votes, 'it-l10n-thumbsup' ) . ' ' . __( 'this.', 'it-l10n-thumbsup' ) . '</span>';
	} elseif ( $up_votes <= 0 && ( $display == 'both' || $display == 'up' ) ) {
		$html .= "<span class='plus'>" . number_format( $up_votes ) . "</span> " . sprintf( "<span id='pb_thumbsup_like_%d'>", absint( $post->ID ) ) .  _n( 'person', 'people', $up_votes, 'it-l10n-thumbsup' ) . ' ' .  _n( 'likes', 'like', $up_votes, 'it-l10n-thumbsup' ) . ' ' . __( 'this.', 'it-l10n-thumbsup' ) . '</span>';
	} elseif ( $down_votes > 0 && ( $display == 'down' ) ) {
		$html .= "<span class='minus'>" . number_format( $down_votes ) . "</span> " . sprintf( "<span id='pb_thumbsup_like_%d'>", absint( $post->ID ) ) .  _n( 'person', 'people', $down_votes, 'it-l10n-thumbsup' ) . ' ' .  _n( 'does not like', 'do not like', $down_votes, 'it-l10n-thumbsup' ) . ' ' . __( 'this.', 'it-l10n-thumbsup' ) . '</span>';
	} elseif ( $display == 'up' ) {
		$html .= "<span class='plus'></span> " . sprintf( "<span id='pb_thumbsup_like_%d'></span>", absint( $post->ID ) );
	} else {
		$html .= "<span class='minus'></span> " . sprintf( "<span id='pb_thumbsup_like_%d'></span>", absint( $post->ID ) );
	}
	$html .= "</div><!--/.pb_thumbsup_vote_stats-->";
	
	$html .= "</div><!--/.pb_thumbsup_item-->". PHP_EOL;

	$html .= "</div><!--/#pb_thumbsup_container-->";
	
	
	//return output
	return $html;
} //end pb_thumbsup_display_posts
?>
